"""
Agent status tracking for AiCippy.
"""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime


@dataclass
class AgentInfo:
    """Information about a single agent."""

    id: str
    type: str
    status: str
    progress: int
    message: str = ""


@dataclass
class StatusInfo:
    """Overall status information."""

    session_id: str | None = None
    connected: bool = False
    active_agents: int = 0
    total_tokens: int = 0
    agents: list[AgentInfo] = field(default_factory=list)


async def get_agent_status() -> StatusInfo:
    """
    Get current agent status information.

    Returns:
        StatusInfo with current session and agent state.
    """
    # This would normally fetch from the active session
    # For now, return placeholder data
    return StatusInfo(
        session_id=None,
        connected=False,
        active_agents=0,
        total_tokens=0,
        agents=[],
    )



"""
Interactive shell for AiCippy.

Provides a rich terminal interface with:
- Live status sidebar with agent visualization
- Task checklist footer
- Keyboard shortcuts
- Beautiful progress indicators
"""

from __future__ import annotations

import asyncio
import signal
import time
from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Callable

from prompt_toolkit import PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.history import FileHistory
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.styles import Style
from rich.align import Align
from rich.console import Console, Group
from rich.live import Live
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from aicippy import __version__
from aicippy.cli.mascot import (
    MascotAnimator,
    MascotMood,
    animate_welcome,
    MASCOT_PRIMARY,
)
from aicippy.cli.ui_components import (
    BRAND_ACCENT,
    BRAND_ERROR,
    BRAND_INFO,
    BRAND_PRIMARY,
    BRAND_SECONDARY,
    BRAND_SUCCESS,
    BRAND_WARNING,
    STATUS_ICONS,
    AgentWidget,
    TaskItem,
    InteractiveLayout,
    create_error_panel,
    create_input_section,
    create_response_panel,
    create_success_panel,
    create_thinking_indicator,
    create_welcome_screen,
)
from aicippy.config import get_settings
from aicippy.utils.correlation import generate_correlation_id
from aicippy.utils.logging import get_logger
from aicippy.cli.upload_manager import UploadManager, get_working_directory
from aicippy.cli.ai_command_executor import AICommandExecutor

logger = get_logger(__name__)
console = Console()


# ============================================================================
# Data Classes
# ============================================================================


@dataclass
class AgentStatus:
    """Status of a single agent."""

    id: str
    type: str
    status: str  # running, thinking, error, idle, complete
    progress: int = 0
    message: str = ""
    started_at: datetime | None = None
    tokens: int = 0

    def to_widget(self) -> AgentWidget:
        """Convert to AgentWidget for UI."""
        return AgentWidget(
            id=self.id,
            name=self.type,
            status=self.status,
            progress=self.progress,
            message=self.message,
            tokens=self.tokens,
        )


@dataclass
class SessionState:
    """Current interactive session state."""

    session_id: str
    model: str = "opus"
    mode: str = "agent"
    agents: list[AgentStatus] = field(default_factory=list)
    tokens_used: int = 0
    connected: bool = False
    conversation: list[dict[str, str]] = field(default_factory=list)
    tasks: list[TaskItem] = field(default_factory=list)
    current_task_idx: int = 0
    mascot_mood: str = "idle"  # idle, thinking, working, happy
    mascot_frame: int = 0
    # New fields for status bar
    session_start: datetime = field(default_factory=datetime.now)
    username: str | None = None
    background_tasks: int = 0

    def add_task(self, content: str) -> None:
        """Add a task to the checklist."""
        self.tasks.append(TaskItem(content=content, status="pending"))

    def start_task(self, idx: int | None = None) -> None:
        """Start a task."""
        idx = idx if idx is not None else self.current_task_idx
        if 0 <= idx < len(self.tasks):
            self.tasks[idx].status = "in_progress"
            self.current_task_idx = idx

    def complete_task(self, idx: int | None = None) -> None:
        """Complete a task."""
        idx = idx if idx is not None else self.current_task_idx
        if 0 <= idx < len(self.tasks):
            self.tasks[idx].status = "completed"
            self.tasks[idx].progress = 100
            if idx == self.current_task_idx and idx + 1 < len(self.tasks):
                self.current_task_idx = idx + 1

    def update_task_progress(self, progress: int, idx: int | None = None) -> None:
        """Update task progress."""
        idx = idx if idx is not None else self.current_task_idx
        if 0 <= idx < len(self.tasks):
            self.tasks[idx].progress = min(100, max(0, progress))

    def get_session_duration(self) -> str:
        """Get session duration as HH:MM:SS string."""
        elapsed = datetime.now() - self.session_start
        total_seconds = int(elapsed.total_seconds())
        hours = total_seconds // 3600
        minutes = (total_seconds % 3600) // 60
        seconds = total_seconds % 60
        return f"{hours:02d}:{minutes:02d}:{seconds:02d}"


# ============================================================================
# Commands
# ============================================================================

# Slash command completions
SLASH_COMMANDS = [
    "/help",
    "/model",
    "/mode",
    "/agents",
    "/kb",
    "/tools",
    "/usage",
    "/history",
    "/clear",
    "/export",
    "/shortcuts",
    "/upload",
    "/uploads",
    "/workspace",
    "/shell",
    "/exec",
    "/quit",
]

MODEL_OPTIONS = ["opus", "sonnet", "llama"]
MODE_OPTIONS = ["agent", "edit", "research", "code"]


# ============================================================================
# Slash Command Handler
# ============================================================================


class SlashCommandHandler:
    """Handler for slash commands in interactive mode."""

    def __init__(self, state: SessionState, console: Console) -> None:
        self.state = state
        self.console = console
        self._upload_manager: UploadManager | None = None
        self._handlers: dict[str, Callable[[list[str]], bool]] = {
            "/help": self._handle_help,
            "/model": self._handle_model,
            "/mode": self._handle_mode,
            "/agents": self._handle_agents,
            "/kb": self._handle_kb,
            "/tools": self._handle_tools,
            "/usage": self._handle_usage,
            "/history": self._handle_history,
            "/clear": self._handle_clear,
            "/export": self._handle_export,
            "/shortcuts": self._handle_shortcuts,
            "/upload": self._handle_upload,
            "/uploads": self._handle_uploads,
            "/workspace": self._handle_workspace,
            "/shell": self._handle_shell,
            "/exec": self._handle_shell,  # Alias for /shell
            "/quit": self._handle_quit,
        }
        self._shell_executor = None

    @property
    def upload_manager(self) -> UploadManager:
        """Get or create upload manager."""
        if self._upload_manager is None:
            self._upload_manager = UploadManager(console=self.console)
        return self._upload_manager

    def handle(self, command: str) -> bool:
        """Handle a slash command. Returns False to exit."""
        parts = command.strip().split()
        if not parts:
            return True

        cmd = parts[0].lower()
        args = parts[1:]

        handler = self._handlers.get(cmd)
        if handler:
            return handler(args)

        self.console.print(create_error_panel(
            f"Unknown command: {cmd}",
            suggestion="Type /help for available commands"
        ))
        return True

    def _handle_help(self, args: list[str]) -> bool:
        """Show help information."""
        help_table = Table(
            title="[bold]Available Commands[/bold]",
            border_style=BRAND_PRIMARY,
            title_style=f"bold {BRAND_INFO}",
            header_style=f"bold {BRAND_ACCENT}",
        )
        help_table.add_column("Command", style=BRAND_INFO)
        help_table.add_column("Description", style="white")

        commands = [
            ("/help", "Show this help message"),
            ("/model <opus|sonnet|llama>", "Switch AI model"),
            ("/mode <agent|edit|research|code>", "Change operation mode"),
            ("/agents spawn <1-10>", "Spawn parallel agents"),
            ("/agents list", "List active agents"),
            ("/agents stop [id|all]", "Stop agent(s)"),
            ("/agents status", "Detailed agent health"),
            ("/upload", "📂 Open file browser to upload files"),
            ("/uploads", "📋 List all uploaded files"),
            ("/workspace", "📁 Show current working directory info"),
            ("/shell <cmd>", "🖥️ Execute shell command with input passthrough"),
            ("/exec <cmd>", "🖥️ Alias for /shell"),
            ("/kb sync", "Push local files to Knowledge Base"),
            ("/kb status", "Knowledge Base status"),
            ("/kb search <query>", "Search Knowledge Base"),
            ("/tools list", "List available tools"),
            ("/tools enable <name>", "Enable a tool"),
            ("/tools disable <name>", "Disable a tool"),
            ("/usage", "Token usage summary"),
            ("/history", "Session history"),
            ("/clear", "Clear conversation"),
            ("/export <json|md>", "Export session"),
            ("/shortcuts", "Show keyboard shortcuts"),
            ("/quit", "Exit AiCippy"),
        ]

        for cmd, desc in commands:
            help_table.add_row(cmd, desc)

        self.console.print(help_table)
        return True

    def _handle_shortcuts(self, args: list[str]) -> bool:
        """Show keyboard shortcuts."""
        from aicippy.cli.ui_components import KEYBOARD_SHORTCUTS

        shortcuts_table = Table(
            title="[bold]Keyboard Shortcuts[/bold]",
            border_style=BRAND_ACCENT,
            title_style=f"bold {BRAND_ACCENT}",
        )
        shortcuts_table.add_column("Key", style=BRAND_ACCENT)
        shortcuts_table.add_column("Action", style="white")

        for key, desc in KEYBOARD_SHORTCUTS:
            shortcuts_table.add_row(key, desc)

        # Add more shortcuts
        extra_shortcuts = [
            ("Ctrl+L", "Clear screen"),
            ("Ctrl+R", "Reverse search history"),
            ("Alt+.", "Insert last argument"),
            ("Esc", "Cancel current input"),
        ]
        for key, desc in extra_shortcuts:
            shortcuts_table.add_row(key, desc)

        self.console.print(shortcuts_table)
        return True

    def _handle_model(self, args: list[str]) -> bool:
        """Switch AI model."""
        if not args:
            current = Text()
            current.append(f"\n {STATUS_ICONS['complete']} ", style=BRAND_SUCCESS)
            current.append("Current model: ", style="dim")
            current.append(self.state.model.upper(), style=f"bold {BRAND_PRIMARY}")
            current.append("\n\n Available: ", style="dim")
            for i, m in enumerate(MODEL_OPTIONS):
                if i > 0:
                    current.append(" │ ", style="dim")
                style = f"bold {BRAND_SUCCESS}" if m == self.state.model else BRAND_INFO
                current.append(m, style=style)
            current.append("\n")
            self.console.print(Panel(current, border_style="dim"))
            return True

        model = args[0].lower()
        if model not in MODEL_OPTIONS:
            self.console.print(create_error_panel(
                f"Invalid model: {model}",
                suggestion=f"Choose from: {', '.join(MODEL_OPTIONS)}"
            ))
            return True

        self.state.model = model
        self.console.print(create_success_panel(
            f"Switched to model: {model.upper()}",
            details="Changes take effect on next query"
        ))
        return True

    def _handle_mode(self, args: list[str]) -> bool:
        """Change operation mode."""
        if not args:
            mode_colors = {
                "agent": BRAND_INFO,
                "edit": BRAND_WARNING,
                "research": BRAND_ACCENT,
                "code": BRAND_SUCCESS,
            }
            current = Text()
            current.append(f"\n {STATUS_ICONS['complete']} ", style=BRAND_SUCCESS)
            current.append("Current mode: ", style="dim")
            current.append(
                self.state.mode.upper(),
                style=f"bold {mode_colors.get(self.state.mode, 'white')}"
            )
            current.append("\n\n Available:\n", style="dim")

            mode_descriptions = {
                "agent": "Multi-agent task orchestration",
                "edit": "Code editing and refactoring",
                "research": "Information gathering and analysis",
                "code": "Code generation and completion",
            }

            for mode, desc in mode_descriptions.items():
                icon = STATUS_ICONS['complete'] if mode == self.state.mode else STATUS_ICONS['pending']
                color = mode_colors.get(mode, 'white')
                current.append(f"   {icon} ", style=color)
                current.append(f"{mode:<10}", style=f"bold {color}")
                current.append(f" {desc}\n", style="dim")

            self.console.print(Panel(current, border_style="dim"))
            return True

        mode = args[0].lower()
        if mode not in MODE_OPTIONS:
            self.console.print(create_error_panel(
                f"Invalid mode: {mode}",
                suggestion=f"Choose from: {', '.join(MODE_OPTIONS)}"
            ))
            return True

        self.state.mode = mode
        self.console.print(create_success_panel(f"Switched to mode: {mode.upper()}"))
        return True

    def _handle_agents(self, args: list[str]) -> bool:
        """Handle agent commands."""
        if not args:
            count = len(self.state.agents)
            self.console.print(Panel(
                Text(f" ⚡ Active agents: {count}/10", style=BRAND_INFO),
                border_style="dim"
            ))
            return True

        subcommand = args[0].lower()

        if subcommand == "spawn":
            count = int(args[1]) if len(args) > 1 else 3
            count = max(1, min(10, count))

            # Create placeholder agents
            for i in range(count):
                agent_id = f"agent-{len(self.state.agents) + 1}"
                self.state.agents.append(AgentStatus(
                    id=agent_id,
                    type=f"specialist-{i + 1}",
                    status="idle",
                    progress=0,
                ))

            self.console.print(create_success_panel(
                f"Spawned {count} agents",
                details="Agents will activate on next query"
            ))
            return True

        elif subcommand == "list":
            if not self.state.agents:
                self.console.print(Panel(
                    Text(" No active agents", style="dim italic"),
                    border_style="dim"
                ))
                return True

            table = Table(
                title="[bold]Active Agents[/bold]",
                border_style=BRAND_SUCCESS,
                title_style=f"bold {BRAND_INFO}",
            )
            table.add_column("ID", style=BRAND_INFO)
            table.add_column("Type", style=BRAND_ACCENT)
            table.add_column("Status")
            table.add_column("Progress")
            table.add_column("Tokens", justify="right")

            for agent in self.state.agents:
                status_color = {
                    "running": BRAND_INFO,
                    "thinking": BRAND_WARNING,
                    "error": BRAND_ERROR,
                    "idle": "dim",
                    "complete": BRAND_SUCCESS,
                }.get(agent.status, "white")

                icon = STATUS_ICONS.get(agent.status, "○")

                # Progress bar
                bar_width = 10
                completed = int(agent.progress / 100 * bar_width)
                bar = "█" * completed + "░" * (bar_width - completed)

                table.add_row(
                    agent.id,
                    agent.type,
                    f"[{status_color}]{icon} {agent.status}[/]",
                    f"[{status_color}]{bar}[/] {agent.progress}%",
                    f"{agent.tokens:,}",
                )

            self.console.print(table)
            return True

        elif subcommand == "stop":
            target = args[1] if len(args) > 1 else "all"
            if target == "all":
                count = len(self.state.agents)
                self.state.agents.clear()
                self.console.print(create_success_panel(
                    f"Stopped all {count} agents"
                ))
            else:
                before = len(self.state.agents)
                self.state.agents = [a for a in self.state.agents if a.id != target]
                if len(self.state.agents) < before:
                    self.console.print(create_success_panel(f"Stopped agent: {target}"))
                else:
                    self.console.print(create_error_panel(f"Agent not found: {target}"))
            return True

        elif subcommand == "status":
            return self._handle_agents(["list"])

        self.console.print(create_error_panel(
            f"Unknown agent subcommand: {subcommand}",
            suggestion="Use: spawn, list, stop, or status"
        ))
        return True

    def _handle_kb(self, args: list[str]) -> bool:
        """Handle Knowledge Base commands."""
        if not args:
            self.console.print(Panel(
                Text(" Usage: /kb <sync|status|search>", style="dim"),
                border_style="dim"
            ))
            return True

        subcommand = args[0].lower()

        if subcommand == "sync":
            self.console.print(create_success_panel(
                "Knowledge Base sync initiated",
                details="Documents will be indexed in the background"
            ))
            return True

        elif subcommand == "status":
            status_table = Table(border_style=BRAND_INFO)
            status_table.add_column("Property", style=BRAND_INFO)
            status_table.add_column("Value", style="white")

            status_table.add_row("Documents", "1,234")
            status_table.add_row("Last Sync", "2 hours ago")
            status_table.add_row("Index Size", "45.2 MB")
            status_table.add_row("Status", f"[{BRAND_SUCCESS}]● Healthy[/]")

            self.console.print(Panel(
                status_table,
                title="[bold]Knowledge Base Status[/bold]",
                border_style=BRAND_INFO,
            ))
            return True

        elif subcommand == "search":
            query = " ".join(args[1:]) if len(args) > 1 else ""
            if not query:
                self.console.print(create_error_panel(
                    "Please provide a search query",
                    suggestion="Example: /kb search AWS Lambda"
                ))
                return True
            self.console.print(Panel(
                Text(f" 🔍 Searching for: {query}", style=BRAND_INFO),
                border_style="dim"
            ))
            return True

        self.console.print(create_error_panel(f"Unknown KB subcommand: {subcommand}"))
        return True

    def _handle_tools(self, args: list[str]) -> bool:
        """Handle tools commands."""
        if not args:
            self.console.print(Panel(
                Text(" Usage: /tools <list|enable|disable>", style="dim"),
                border_style="dim"
            ))
            return True

        subcommand = args[0].lower()

        if subcommand == "list":
            table = Table(
                title="[bold]Available Tools[/bold]",
                border_style=BRAND_INFO,
            )
            table.add_column("Tool", style=BRAND_INFO)
            table.add_column("Status")
            table.add_column("Category", style="dim")
            table.add_column("Description")

            tools = [
                ("aws", True, "Cloud", "AWS CLI operations"),
                ("gcloud", True, "Cloud", "Google Cloud CLI"),
                ("gh", True, "VCS", "GitHub CLI"),
                ("firebase", True, "Cloud", "Firebase CLI"),
                ("figma", True, "Design", "Figma API"),
                ("drive", True, "Storage", "Google Drive API"),
                ("gmail", True, "Comm", "Gmail API"),
                ("canva", False, "Design", "Canva API"),
                ("azure", False, "Cloud", "Azure CLI"),
                ("razorpay", True, "Payment", "Razorpay API"),
                ("paypal", False, "Payment", "PayPal API"),
                ("stripe", True, "Payment", "Stripe CLI"),
                ("bash", True, "Shell", "Shell commands"),
            ]

            for name, enabled, category, desc in tools:
                status = f"[{BRAND_SUCCESS}]{STATUS_ICONS['complete']} Enabled[/]" if enabled else f"[dim]{STATUS_ICONS['pending']} Disabled[/]"
                table.add_row(name, status, category, desc)

            self.console.print(table)
            return True

        elif subcommand in ("enable", "disable"):
            tool_name = args[1] if len(args) > 1 else None
            if not tool_name:
                self.console.print(create_error_panel("Please specify a tool name"))
                return True

            action = "Enabled" if subcommand == "enable" else "Disabled"
            self.console.print(create_success_panel(f"{action} tool: {tool_name}"))
            return True

        self.console.print(create_error_panel(f"Unknown tools subcommand: {subcommand}"))
        return True

    def _handle_usage(self, args: list[str]) -> bool:
        """Show token usage."""
        table = Table(
            title="[bold]Token Usage[/bold]",
            border_style=BRAND_WARNING,
        )
        table.add_column("Metric", style=BRAND_INFO)
        table.add_column("Value", style=BRAND_WARNING, justify="right")

        table.add_row("Session Tokens", f"{self.state.tokens_used:,}")
        table.add_row("Model", self.state.model.upper())
        table.add_row("Active Agents", str(len(self.state.agents)))
        table.add_row("Messages", str(len(self.state.conversation)))

        self.console.print(table)
        return True

    def _handle_history(self, args: list[str]) -> bool:
        """Show conversation history."""
        if not self.state.conversation:
            self.console.print(Panel(
                Text(" No conversation history", style="dim italic"),
                border_style="dim"
            ))
            return True

        for i, msg in enumerate(self.state.conversation[-10:], 1):
            role = msg.get("role", "unknown")
            content = msg.get("content", "")[:100]
            role_color = BRAND_INFO if role == "user" else BRAND_SUCCESS
            icon = "❯" if role == "user" else "◈"

            line = Text()
            line.append(f" {icon} ", style=role_color)
            line.append(f"{role}: ", style=f"bold {role_color}")
            line.append(content, style="white")
            if len(msg.get("content", "")) > 100:
                line.append("...", style="dim")
            self.console.print(line)

        return True

    def _handle_clear(self, args: list[str]) -> bool:
        """Clear conversation."""
        self.state.conversation.clear()
        self.state.tasks.clear()
        self.state.current_task_idx = 0
        self.console.clear()
        self.console.print(create_success_panel("Conversation and tasks cleared"))
        return True

    def _handle_export(self, args: list[str]) -> bool:
        """Export session."""
        import json

        format_type = args[0] if args else "json"

        if format_type not in ("json", "md"):
            self.console.print(create_error_panel(
                "Invalid format",
                suggestion="Use 'json' or 'md'"
            ))
            return True

        filename = f"aicippy_session_{datetime.now().strftime('%Y%m%d_%H%M%S')}.{format_type}"

        if format_type == "json":
            with open(filename, "w") as f:
                json.dump(
                    {
                        "session_id": self.state.session_id,
                        "model": self.state.model,
                        "mode": self.state.mode,
                        "tokens_used": self.state.tokens_used,
                        "conversation": self.state.conversation,
                    },
                    f,
                    indent=2,
                )
        else:
            with open(filename, "w") as f:
                f.write("# AiCippy Session Export\n\n")
                f.write(f"**Session:** {self.state.session_id}\n")
                f.write(f"**Model:** {self.state.model}\n")
                f.write(f"**Tokens:** {self.state.tokens_used:,}\n\n")
                f.write("## Conversation\n\n")
                for msg in self.state.conversation:
                    role = msg.get("role", "unknown")
                    content = msg.get("content", "")
                    f.write(f"### {role.title()}\n\n{content}\n\n")

        self.console.print(create_success_panel(
            f"Session exported",
            details=f"Saved to: {filename}"
        ))
        return True

    def _handle_upload(self, args: list[str]) -> bool:
        """
        Handle /upload command - opens OS file browser dialog.

        Usage:
            /upload             - Open file browser for current uploads folder
            /upload <path>      - Upload to specific folder
        """
        from pathlib import Path

        target_dir = None
        if args:
            target_path = Path(args[0]).expanduser()
            if target_path.is_dir():
                target_dir = target_path
            else:
                # Create if doesn't exist
                target_dir = target_path
                target_dir.mkdir(parents=True, exist_ok=True)

        try:
            results = self.upload_manager.prompt_upload(target_dir=target_dir)

            if results:
                successful = [r for r in results if r.success]
                if successful:
                    # Notify about uploaded files for AI context
                    file_list = ", ".join(r.file_name for r in successful)
                    self.console.print(Panel(
                        Text(
                            f"💡 Tip: Reference uploaded files in your prompts.\n"
                            f"   Example: \"Use the uploaded logo {successful[0].file_name} to...\"",
                            style="dim italic",
                        ),
                        border_style="dim",
                    ))

        except Exception as e:
            self.console.print(create_error_panel(
                f"Upload failed: {e}",
                suggestion="Check file permissions and try again"
            ))

        return True

    def _handle_uploads(self, args: list[str]) -> bool:
        """
        Handle /uploads command - list uploaded files.

        Usage:
            /uploads            - List all uploaded files
            /uploads clear      - Clear all uploads
        """
        if args and args[0].lower() == "clear":
            if self.upload_manager.clear_uploads():
                self.console.print(create_success_panel("All uploads cleared"))
            else:
                self.console.print(create_error_panel("Failed to clear uploads"))
            return True

        self.upload_manager.show_uploads_list()
        return True

    def _handle_workspace(self, args: list[str]) -> bool:
        """
        Handle /workspace command - show working directory info.

        Usage:
            /workspace          - Show current workspace info
        """
        self.upload_manager.show_working_directory()
        return True

    def _handle_shell(self, args: list[str]) -> bool:
        """
        Handle /shell command - execute shell command with input passthrough.

        Usage:
            /shell <command>    - Execute command with interactive input support
            /exec <command>     - Alias for /shell

        Examples:
            /shell git status
            /shell npm install
            /exec pip install requests
        """
        if not args:
            self.console.print(Panel(
                Text(
                    "Usage: /shell <command>\n\n"
                    "Execute shell commands with interactive input passthrough.\n"
                    "Any prompts from the command will be displayed for you to respond.\n\n"
                    "Examples:\n"
                    "  /shell git status\n"
                    "  /shell npm install\n"
                    "  /shell pip install requests\n"
                    "  /exec docker login",
                    style=BRAND_INFO,
                ),
                title=f"[bold {BRAND_ACCENT}]Shell Command[/bold {BRAND_ACCENT}]",
                border_style=BRAND_ACCENT,
            ))
            return True

        command = " ".join(args)

        try:
            from aicippy.cli.shell_executor import ShellExecutor, is_interactive_command

            if self._shell_executor is None:
                self._shell_executor = ShellExecutor(
                    console=self.console,
                    working_dir=get_working_directory(),
                )

            # Check if command is interactive
            if is_interactive_command(command):
                # Run interactive command with input callback
                def get_input(prompt: str) -> str:
                    return self._shell_executor.prompt_user_input(prompt)

                # Use asyncio to run the interactive command
                import asyncio
                loop = asyncio.get_event_loop()
                if loop.is_running():
                    # We're already in an async context, schedule as task
                    result = asyncio.run_coroutine_threadsafe(
                        self._shell_executor.execute_interactive(
                            command,
                            input_callback=get_input,
                        ),
                        loop,
                    ).result()
                else:
                    result = asyncio.run(
                        self._shell_executor.execute_interactive(
                            command,
                            input_callback=get_input,
                        )
                    )
            else:
                # Simple command execution
                result = self._shell_executor.execute_simple(command)

            # Store output in conversation for AI context
            if result.stdout:
                self.state.conversation.append({
                    "role": "system",
                    "content": f"[Shell Command Output]\n$ {command}\n{result.stdout[:2000]}",
                })

            # Display result
            self._shell_executor.display_result(result)

        except Exception as e:
            self.console.print(create_error_panel(
                f"Shell execution failed: {e}",
                suggestion="Check the command syntax and try again"
            ))

        return True

    def _handle_quit(self, args: list[str]) -> bool:
        """Exit the session."""
        return False


# ============================================================================
# Main Interactive Session
# ============================================================================


async def start_interactive_session() -> None:
    """Start the interactive AiCippy session with beautiful UI."""
    settings = get_settings()

    # Generate session ID
    session_id = generate_correlation_id()

    # Initialize state
    state = SessionState(
        session_id=session_id,
        model=settings.default_model,
    )

    # Add initial demo tasks
    state.add_task("Initialize session")
    state.start_task(0)

    # Create command handler
    cmd_handler = SlashCommandHandler(state, console)

    # Create the new interactive layout
    layout = InteractiveLayout(console, __version__)

    # Create prompt session with history
    history_file = settings.local_config_dir / "history"
    history_file.parent.mkdir(parents=True, exist_ok=True)

    # Custom prompt style - use cyan for input text (visible on dark terminals)
    prompt_style = Style.from_dict({
        "prompt": f"{BRAND_PRIMARY} bold",
        "": "#00d7ff",  # Bright cyan for user input text
    })

    # Key bindings
    kb = KeyBindings()

    @kb.add(Keys.ControlL)
    def clear_screen(event: Any) -> None:
        """Clear the screen and preview."""
        console.clear()
        layout.clear_preview()

    completer = WordCompleter(
        SLASH_COMMANDS + MODEL_OPTIONS + MODE_OPTIONS + ["quit", "exit", "q", "bye"],
        ignore_case=True,
    )

    session: PromptSession[str] = PromptSession(
        history=FileHistory(str(history_file)),
        auto_suggest=AutoSuggestFromHistory(),
        completer=completer,
        style=prompt_style,
        key_bindings=kb,
    )

    # Display animated welcome screen with Cippy mascot
    console.clear()
    try:
        await animate_welcome(console, __version__, duration=2.0)
    except Exception:
        # Fallback to static welcome if animation fails
        console.print(create_welcome_screen(__version__))

    # Add initialization messages to preview
    layout.add_success("AiCippy initialized")
    layout.add_preview_line(f"Session ID: {session_id[:8]}...", style="dim")

    # Complete initialization task
    state.complete_task(0)

    # Check authentication and get username
    try:
        from aicippy.auth.cognito import CognitoAuth
        from aicippy.installer.session_manager import get_current_user
        auth = CognitoAuth()
        if auth.is_authenticated():
            state.connected = True
            # Get username from session
            username, email = get_current_user()
            state.username = username or email or "authenticated"
            layout.add_success(f"Authenticated as {state.username}")
        else:
            layout.add_preview_line("Not authenticated. Run 'aicippy login' first.", style=BRAND_WARNING)
    except Exception:
        pass

    # Display working directory information
    working_dir = get_working_directory()
    layout.add_preview_line(f"Working Directory: {working_dir}", style=BRAND_INFO)

    # Check and request necessary permissions
    try:
        uploads_dir = working_dir / "uploads"
        if not uploads_dir.exists():
            uploads_dir.mkdir(parents=True, exist_ok=True)
            layout.add_success(f"Created uploads folder")

        # Test write permission
        test_file = uploads_dir / ".aicippy_test"
        test_file.write_text("test")
        test_file.unlink()
        layout.add_success("Write permissions verified")
    except PermissionError:
        layout.add_error(f"Permission denied for uploads folder")
    except Exception:
        pass

    # Main loop
    running = True

    def signal_handler(sig: int, frame: Any) -> None:
        nonlocal running
        layout.add_preview_line("Type 'quit' to exit", style=BRAND_WARNING)

    signal.signal(signal.SIGINT, signal_handler)

    while running:
        try:
            # Increment mascot animation frame
            state.mascot_frame = (state.mascot_frame + 1) % 100

            # Update and display the layout
            updated_layout = layout.update(
                agents=[a.to_widget() for a in state.agents],
                tasks=state.tasks,
                current_task_idx=state.current_task_idx,
                session_model=state.model,
                tokens_used=state.tokens_used,
                username=state.username,
                session_duration=state.get_session_duration(),
                background_tasks=state.background_tasks,
                connected=state.connected,
                mascot_mood=state.mascot_mood,
            )
            console.print(updated_layout)

            # Create input prompt
            prompt_text = create_input_section(mode=state.mode, model=state.model)
            console.print(prompt_text, end="")

            # Get input
            user_input = await asyncio.get_event_loop().run_in_executor(
                None,
                lambda: session.prompt(""),
            )

            if not user_input.strip():
                continue

            # Handle quit/exit commands without slash prefix
            user_input_lower = user_input.strip().lower()
            if user_input_lower in ("quit", "exit", "q", "bye"):
                running = False
                continue

            # Handle slash commands
            if user_input.startswith("/"):
                running = cmd_handler.handle(user_input)
                continue

            # Log user input to preview
            layout.add_preview_line(f"❯ {user_input}", style=BRAND_INFO)

            # Regular chat message
            state.conversation.append({"role": "user", "content": user_input})

            # Add task for this query
            state.add_task(f"Process: {user_input[:30]}...")
            state.start_task(len(state.tasks) - 1)

            # Process with orchestrator
            try:
                from aicippy.agents.orchestrator import AgentOrchestrator

                orchestrator = AgentOrchestrator(
                    model_id=settings.get_model_id(state.model),
                    max_agents=len(state.agents) if state.agents else 3,
                )

                # Update agents status and mascot mood
                state.mascot_mood = "thinking"
                for agent in state.agents:
                    agent.status = "thinking"
                    agent.progress = 0
                    layout.add_agent_update(agent.id, "thinking", "Processing query...")

                layout.add_preview_line("◐ Thinking...", style=BRAND_WARNING)

                # Show animated thinking indicator with live progress
                start_time = time.monotonic()
                animation_running = True
                frame_idx = 0

                async def process_with_animation() -> Any:
                    """Process the chat request with animated indicator."""
                    nonlocal animation_running, frame_idx
                    state.mascot_mood = "working"

                    with Live(
                        create_thinking_indicator(frame_idx),
                        console=console,
                        transient=True,
                        refresh_per_second=4,
                    ) as live:
                        async def update_display() -> None:
                            """Update the live display with new animation frames."""
                            nonlocal frame_idx
                            while animation_running:
                                frame_idx += 1
                                state.mascot_frame = frame_idx
                                # Update progress for agents (simulate)
                                for agent in state.agents:
                                    if agent.status == "thinking":
                                        agent.progress = min(95, agent.progress + 2)
                                # Update the live display
                                live.update(create_thinking_indicator(frame_idx))
                                await asyncio.sleep(0.25)

                        # Start animation task
                        animation_task = asyncio.create_task(update_display())

                        try:
                            # Process the actual chat request
                            result = await orchestrator.chat(user_input)
                            return result
                        finally:
                            # Stop animation
                            animation_running = False
                            animation_task.cancel()
                            try:
                                await animation_task
                            except asyncio.CancelledError:
                                pass

                response = await process_with_animation()

                # Mark agents complete
                for agent in state.agents:
                    agent.status = "complete"
                    agent.progress = 100
                    layout.add_agent_update(agent.id, "complete", "Done")

                execution_time = time.monotonic() - start_time

                # Add response to preview
                layout.add_success(f"Response received ({execution_time:.1f}s)")

                # Display response with beautiful panel
                console.print(create_response_panel(
                    content=response.content,
                    agent_type=response.agent_type.value if response.agent_type else None,
                    tokens_used=(
                        (response.usage.input_tokens + response.usage.output_tokens)
                        if response.usage else 0
                    ),
                    execution_time=execution_time,
                ))

                # Update state
                state.conversation.append({"role": "assistant", "content": response.content})
                if response.usage:
                    state.tokens_used += response.usage.input_tokens + response.usage.output_tokens
                    layout.add_preview_line(
                        f"Tokens: +{response.usage.input_tokens + response.usage.output_tokens:,}",
                        style="dim"
                    )

                # AUTO-EXECUTE: Detect and offer to execute commands from AI response
                try:
                    ai_cmd_executor = AICommandExecutor(
                        console=console,
                        working_dir=get_working_directory(),
                        auto_confirm=False,  # Always ask for confirmation
                    )

                    detected_commands = ai_cmd_executor.detect_commands(response.content)

                    if detected_commands:
                        layout.add_preview_line(f"Detected {len(detected_commands)} command(s)", style=BRAND_INFO)

                        # Execute detected commands with passthrough
                        cmd_results = await ai_cmd_executor.execute_detected_commands(
                            response.content,
                            execute_all=False,
                        )

                        # Add execution results to conversation for AI context
                        execution_summary = ai_cmd_executor.get_execution_summary()
                        if execution_summary:
                            state.conversation.append({
                                "role": "system",
                                "content": f"[Command Execution Results]\n{execution_summary}",
                            })
                            layout.add_success("Command results added to context")

                except Exception as cmd_err:
                    logger.warning("ai_command_execution_warning", error=str(cmd_err))
                    # Don't fail the conversation if command detection/execution fails

                # Complete task
                state.complete_task()

                # Set mascot to happy after success
                state.mascot_mood = "happy"

                # Reset agents to idle
                for agent in state.agents:
                    agent.status = "idle"

                # Reset mascot to idle after a short happy moment
                await asyncio.sleep(0.5)
                state.mascot_mood = "idle"

            except Exception as e:
                logger.exception("chat_error", error=str(e))
                layout.add_error(str(e))
                console.print(create_error_panel(
                    str(e),
                    suggestion="Check your connection and try again"
                ))

                # Mark task as failed
                if state.tasks and state.current_task_idx < len(state.tasks):
                    state.tasks[state.current_task_idx].status = "pending"

        except EOFError:
            running = False
        except KeyboardInterrupt:
            continue

    # Farewell message with waving mascot
    console.print()

    # Create farewell with mascot waving goodbye
    animator = MascotAnimator(mood=MascotMood.WAVING, use_mini=True)
    frames = animator.get_frames()
    farewell_mascot = animator.render_frame(frames[0])

    farewell_text = Text()
    farewell_text.append("\n")
    farewell_text.append(f" {STATUS_ICONS['complete']} ", style=BRAND_SUCCESS)
    farewell_text.append("Session ended. ", style=f"bold {BRAND_PRIMARY}")
    farewell_text.append("Goodbye!", style=f"bold {MASCOT_PRIMARY}")
    farewell_text.append(f"\n\n   Tokens used: {state.tokens_used:,}", style="dim")
    farewell_text.append(f"\n   Messages: {len(state.conversation)}", style="dim")
    farewell_text.append("\n\n   See you next time!", style=f"italic {BRAND_ACCENT}")

    console.print(Panel(
        Group(
            Align.center(farewell_mascot),
            farewell_text,
        ),
        border_style=BRAND_PRIMARY,
        title="[bold]Bye! 👋[/bold]",
        title_align="center",
    ))



"""
Shell Executor with User Input Passthrough for AiCippy CLI.

Provides intelligent command execution with:
- Real-time output streaming
- User input passthrough for interactive commands
- Cross-platform support (Windows/macOS/Linux)
- Command output capture for AI processing
"""

from __future__ import annotations

import asyncio
import os
import signal
import subprocess
import sys
import threading
from dataclasses import dataclass, field
from pathlib import Path
from queue import Queue, Empty
from typing import Callable, Final, IO

# Unix-only imports - handle gracefully on Windows
if os.name != "nt":
    import pty
    import select
    import fcntl
else:
    pty = None  # type: ignore[assignment]
    select = None  # type: ignore[assignment]
    fcntl = None  # type: ignore[assignment]

from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.live import Live
from rich.spinner import Spinner

# Brand colors
BRAND_PRIMARY: Final[str] = "#667eea"
BRAND_SUCCESS: Final[str] = "#10b981"
BRAND_WARNING: Final[str] = "#f59e0b"
BRAND_ERROR: Final[str] = "#ef4444"
BRAND_INFO: Final[str] = "#3b82f6"
BRAND_ACCENT: Final[str] = "#a78bfa"


@dataclass
class CommandResult:
    """Result of command execution."""

    success: bool
    exit_code: int
    stdout: str
    stderr: str
    command: str
    duration: float = 0.0
    was_interrupted: bool = False
    user_inputs: list[str] = field(default_factory=list)


@dataclass
class InputRequest:
    """Request for user input from a command."""

    prompt: str
    is_password: bool = False
    default: str | None = None


def detect_shell() -> str:
    """Detect the user's shell."""
    if os.name == "nt":
        # Windows - check for PowerShell or cmd
        if os.environ.get("PSModulePath"):
            return "powershell"
        return "cmd"

    # Unix - use SHELL environment variable
    return os.environ.get("SHELL", "/bin/sh")


def is_interactive_command(command: str) -> bool:
    """
    Check if a command is likely to request user input.

    Args:
        command: The command to check.

    Returns:
        True if the command might be interactive.
    """
    interactive_patterns = [
        "sudo ",
        "ssh ",
        "scp ",
        "git push",
        "git pull",
        "git clone",
        "npm login",
        "npm publish",
        "docker login",
        "aws configure",
        "gcloud auth",
        "az login",
        "read ",
        "passwd",
        "mysql ",
        "psql ",
        "mongo ",
        "redis-cli",
        " -i",
        "--interactive",
        "rm -i",
        "cp -i",
        "mv -i",
        "pip install",  # Can prompt for confirmation
        "brew install",
        "apt install",
        "apt-get install",
        "yum install",
        "dnf install",
    ]

    command_lower = command.lower()
    return any(pattern in command_lower for pattern in interactive_patterns)


class ShellExecutor:
    """
    Executes shell commands with user input passthrough.

    Provides real-time output streaming and handles interactive
    commands that require user input.
    """

    def __init__(
        self,
        console: Console | None = None,
        working_dir: Path | None = None,
    ) -> None:
        """
        Initialize shell executor.

        Args:
            console: Rich console for output.
            working_dir: Working directory for commands.
        """
        self.console = console or Console()
        self.working_dir = working_dir or Path.cwd()
        self._shell = detect_shell()
        self._output_queue: Queue[str] = Queue()
        self._current_process: subprocess.Popen | None = None

    def _read_output_thread(
        self,
        stream: IO[str],
        queue: Queue[str],
        prefix: str = "",
    ) -> None:
        """Thread function to read output from a stream."""
        try:
            for line in iter(stream.readline, ""):
                if line:
                    queue.put(f"{prefix}{line}")
        except Exception:
            pass
        finally:
            try:
                stream.close()
            except Exception:
                pass

    def execute_simple(
        self,
        command: str,
        timeout: int = 300,
        capture_output: bool = True,
    ) -> CommandResult:
        """
        Execute a simple non-interactive command.

        Args:
            command: Command to execute.
            timeout: Timeout in seconds.
            capture_output: Whether to capture output.

        Returns:
            CommandResult with execution details.
        """
        import time
        start_time = time.monotonic()

        try:
            # Determine shell command format
            if os.name == "nt":
                if self._shell == "powershell":
                    shell_cmd = ["powershell", "-NoProfile", "-Command", command]
                else:
                    shell_cmd = ["cmd", "/c", command]
                use_shell = False
            else:
                shell_cmd = command
                use_shell = True

            result = subprocess.run(
                shell_cmd,
                shell=use_shell,
                cwd=str(self.working_dir),
                capture_output=capture_output,
                text=True,
                timeout=timeout,
            )

            duration = time.monotonic() - start_time

            return CommandResult(
                success=result.returncode == 0,
                exit_code=result.returncode,
                stdout=result.stdout or "",
                stderr=result.stderr or "",
                command=command,
                duration=duration,
            )

        except subprocess.TimeoutExpired:
            return CommandResult(
                success=False,
                exit_code=-1,
                stdout="",
                stderr="Command timed out",
                command=command,
                duration=timeout,
            )
        except Exception as e:
            return CommandResult(
                success=False,
                exit_code=-1,
                stdout="",
                stderr=str(e),
                command=command,
                duration=time.monotonic() - start_time,
            )

    async def execute_interactive(
        self,
        command: str,
        timeout: int = 300,
        input_callback: Callable[[str], str] | None = None,
    ) -> CommandResult:
        """
        Execute an interactive command with user input passthrough.

        Args:
            command: Command to execute.
            timeout: Timeout in seconds.
            input_callback: Callback for getting user input.

        Returns:
            CommandResult with execution details.
        """
        import time
        start_time = time.monotonic()
        collected_output = []
        collected_stderr = []
        user_inputs = []

        # Show command being executed
        self.console.print(Panel(
            Text(f"$ {command}", style=BRAND_INFO),
            title=f"[bold {BRAND_ACCENT}]Executing Command[/bold {BRAND_ACCENT}]",
            border_style=BRAND_ACCENT,
        ))

        try:
            if os.name == "nt":
                # Windows: Use subprocess with pipes
                return await self._execute_windows_interactive(
                    command, timeout, input_callback
                )

            # Unix: Use pseudo-terminal for better interactivity
            return await self._execute_unix_interactive(
                command, timeout, input_callback
            )

        except Exception as e:
            return CommandResult(
                success=False,
                exit_code=-1,
                stdout="".join(collected_output),
                stderr=str(e),
                command=command,
                duration=time.monotonic() - start_time,
                user_inputs=user_inputs,
            )

    async def _execute_windows_interactive(
        self,
        command: str,
        timeout: int,
        input_callback: Callable[[str], str] | None,
    ) -> CommandResult:
        """Execute interactive command on Windows."""
        import time
        start_time = time.monotonic()
        collected_output = []
        collected_stderr = []
        user_inputs = []

        if self._shell == "powershell":
            shell_cmd = ["powershell", "-NoProfile", "-Command", command]
        else:
            shell_cmd = ["cmd", "/c", command]

        process = await asyncio.create_subprocess_exec(
            *shell_cmd,
            stdin=asyncio.subprocess.PIPE,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
            cwd=str(self.working_dir),
        )

        self._current_process = process

        async def read_stream(stream, collection, prefix=""):
            """Read from stream and display output."""
            while True:
                line = await stream.readline()
                if not line:
                    break
                decoded = line.decode("utf-8", errors="replace")
                collection.append(decoded)
                self.console.print(Text(decoded.rstrip(), style="white"))

                # Check for input prompts
                if input_callback and self._looks_like_prompt(decoded):
                    user_input = input_callback(decoded)
                    if user_input is not None:
                        user_inputs.append(user_input)
                        process.stdin.write((user_input + "\n").encode())
                        await process.stdin.drain()

        try:
            await asyncio.wait_for(
                asyncio.gather(
                    read_stream(process.stdout, collected_output),
                    read_stream(process.stderr, collected_stderr, "[stderr] "),
                ),
                timeout=timeout,
            )

            await process.wait()
            exit_code = process.returncode

        except asyncio.TimeoutError:
            process.kill()
            return CommandResult(
                success=False,
                exit_code=-1,
                stdout="".join(collected_output),
                stderr="Command timed out",
                command=command,
                duration=timeout,
                user_inputs=user_inputs,
            )

        return CommandResult(
            success=exit_code == 0,
            exit_code=exit_code,
            stdout="".join(collected_output),
            stderr="".join(collected_stderr),
            command=command,
            duration=time.monotonic() - start_time,
            user_inputs=user_inputs,
        )

    async def _execute_unix_interactive(
        self,
        command: str,
        timeout: int,
        input_callback: Callable[[str], str] | None,
    ) -> CommandResult:
        """Execute interactive command on Unix using PTY."""
        import time
        start_time = time.monotonic()
        collected_output = []
        user_inputs = []

        # Check if PTY is available
        if pty is None or select is None:
            # Fallback to Windows-style execution
            return await self._execute_windows_interactive(
                command, timeout, input_callback
            )

        # Create pseudo-terminal
        master_fd, slave_fd = pty.openpty()

        try:
            process = subprocess.Popen(
                command,
                shell=True,
                stdin=slave_fd,
                stdout=slave_fd,
                stderr=slave_fd,
                cwd=str(self.working_dir),
                preexec_fn=os.setsid,
            )

            os.close(slave_fd)
            self._current_process = process

            # Set non-blocking mode
            if fcntl is not None:
                flags = fcntl.fcntl(master_fd, fcntl.F_GETFL)
                fcntl.fcntl(master_fd, fcntl.F_SETFL, flags | os.O_NONBLOCK)

            end_time = time.monotonic() + timeout
            output_buffer = ""

            while process.poll() is None:
                if time.monotonic() > end_time:
                    os.killpg(os.getpgid(process.pid), signal.SIGTERM)
                    return CommandResult(
                        success=False,
                        exit_code=-1,
                        stdout="".join(collected_output),
                        stderr="Command timed out",
                        command=command,
                        duration=timeout,
                        user_inputs=user_inputs,
                    )

                # Read available output
                try:
                    readable, _, _ = select.select([master_fd], [], [], 0.1)
                    if readable:
                        data = os.read(master_fd, 1024)
                        if data:
                            decoded = data.decode("utf-8", errors="replace")
                            collected_output.append(decoded)
                            output_buffer += decoded

                            # Display output
                            self.console.print(Text(decoded, style="white"), end="")

                            # Check for input prompts
                            if input_callback and self._looks_like_prompt(output_buffer):
                                user_input = input_callback(output_buffer)
                                if user_input is not None:
                                    user_inputs.append(user_input)
                                    os.write(master_fd, (user_input + "\n").encode())
                                    output_buffer = ""
                except (OSError, IOError):
                    await asyncio.sleep(0.05)

            # Read any remaining output
            try:
                while True:
                    readable, _, _ = select.select([master_fd], [], [], 0.1)
                    if not readable:
                        break
                    data = os.read(master_fd, 1024)
                    if not data:
                        break
                    decoded = data.decode("utf-8", errors="replace")
                    collected_output.append(decoded)
                    self.console.print(Text(decoded, style="white"), end="")
            except (OSError, IOError):
                pass

            exit_code = process.returncode

        finally:
            os.close(master_fd)

        return CommandResult(
            success=exit_code == 0,
            exit_code=exit_code,
            stdout="".join(collected_output),
            stderr="",
            command=command,
            duration=time.monotonic() - start_time,
            user_inputs=user_inputs,
        )

    def _looks_like_prompt(self, text: str) -> bool:
        """
        Check if text looks like an input prompt.

        Args:
            text: Text to check.

        Returns:
            True if it looks like a prompt.
        """
        prompt_indicators = [
            "password:",
            "Password:",
            "PASSWORD:",
            "passphrase:",
            "Enter ",
            "enter ",
            "Type ",
            "type ",
            "(y/n)",
            "(Y/N)",
            "[y/N]",
            "[Y/n]",
            "[yes/no]",
            "? ",
            ": $",
            ">> ",
            "> ",
            "Username:",
            "username:",
            "Email:",
            "email:",
            "Token:",
            "token:",
            "API key:",
            "api key:",
        ]

        return any(indicator in text for indicator in prompt_indicators)

    def prompt_user_input(
        self,
        prompt_text: str,
        is_password: bool = False,
    ) -> str:
        """
        Prompt user for input in the CLI.

        Args:
            prompt_text: The prompt text to display.
            is_password: Whether to hide input (for passwords).

        Returns:
            User's input string.
        """
        from prompt_toolkit import prompt
        from prompt_toolkit.styles import Style

        style = Style.from_dict({
            "prompt": f"{BRAND_ACCENT} bold",
        })

        self.console.print(Panel(
            Text(prompt_text.strip(), style=BRAND_INFO),
            title=f"[bold {BRAND_WARNING}]Input Required[/bold {BRAND_WARNING}]",
            border_style=BRAND_WARNING,
        ))

        try:
            user_input = prompt(
                "➤ ",
                style=style,
                is_password=is_password,
            )
            return user_input
        except (EOFError, KeyboardInterrupt):
            return ""

    def interrupt_current(self) -> bool:
        """
        Interrupt the currently running command.

        Returns:
            True if a command was interrupted.
        """
        if self._current_process and self._current_process.poll() is None:
            try:
                if os.name == "nt":
                    self._current_process.terminate()
                else:
                    os.killpg(os.getpgid(self._current_process.pid), signal.SIGINT)
                return True
            except Exception:
                pass
        return False

    def display_result(self, result: CommandResult) -> None:
        """
        Display command result in a nice format.

        Args:
            result: CommandResult to display.
        """
        if result.success:
            status_text = Text()
            status_text.append("✓ ", style=BRAND_SUCCESS)
            status_text.append(f"Command completed ", style=BRAND_SUCCESS)
            status_text.append(f"(exit code: {result.exit_code}, ", style="dim")
            status_text.append(f"duration: {result.duration:.2f}s)", style="dim")

            self.console.print(Panel(
                status_text,
                border_style=BRAND_SUCCESS,
            ))
        else:
            status_text = Text()
            status_text.append("✗ ", style=BRAND_ERROR)
            status_text.append(f"Command failed ", style=BRAND_ERROR)
            status_text.append(f"(exit code: {result.exit_code})", style="dim")

            error_content = [status_text]

            if result.stderr:
                error_content.append(Text("\n\nError output:", style=BRAND_WARNING))
                error_content.append(Text(result.stderr[:500], style="white"))

            from rich.console import Group
            self.console.print(Panel(
                Group(*error_content),
                border_style=BRAND_ERROR,
            ))


async def execute_command_with_passthrough(
    command: str,
    console: Console | None = None,
    working_dir: Path | None = None,
    timeout: int = 300,
) -> CommandResult:
    """
    Convenience function to execute a command with input passthrough.

    Args:
        command: Command to execute.
        console: Rich console for output.
        working_dir: Working directory.
        timeout: Timeout in seconds.

    Returns:
        CommandResult with execution details.
    """
    executor = ShellExecutor(console=console, working_dir=working_dir)

    # Check if command is likely to be interactive
    if is_interactive_command(command):
        # Use interactive execution with input callback
        def get_input(prompt: str) -> str:
            return executor.prompt_user_input(prompt)

        result = await executor.execute_interactive(
            command,
            timeout=timeout,
            input_callback=get_input,
        )
    else:
        # Use simple execution
        result = executor.execute_simple(command, timeout=timeout)

    executor.display_result(result)
    return result



"""
Dependency Manager for AiCippy Installer.

Handles automatic detection and installation of required dependencies
across Windows, macOS, and Linux platforms.
"""

from __future__ import annotations

import json
import subprocess
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import Callable, Final

from aicippy.installer.platform_detect import (
    OperatingSystem,
    PlatformInfo,
    run_shell_command,
)

# Required Python packages with minimum versions
REQUIRED_PACKAGES: Final[dict[str, str]] = {
    "pydantic": "2.6.0",
    "pydantic-settings": "2.2.0",
    "rich": "13.7.0",
    "typer": "0.12.0",
    "boto3": "1.34.0",
    "botocore": "1.34.0",
    "websockets": "12.0",
    "httpx": "0.27.0",
    "anyio": "4.3.0",
    "keyring": "25.0.0",
    "python-jose": "3.3.0",
    "aiofiles": "23.2.0",
    "tenacity": "8.2.0",
    "structlog": "24.1.0",
    "orjson": "3.9.0",
    "prompt-toolkit": "3.0.43",
    "feedparser": "6.0.11",
    "beautifulsoup4": "4.12.0",
    "lxml": "5.1.0",
    "markdown": "3.5.0",
    "cryptography": "42.0.0",
}

# Optional packages for enhanced features
OPTIONAL_PACKAGES: Final[dict[str, str]] = {
    "uvloop": "0.19.0",  # Unix only - faster event loop
}

# System dependencies per OS (package manager -> packages)
SYSTEM_DEPENDENCIES: Final[dict[OperatingSystem, dict[str, list[str]]]] = {
    OperatingSystem.MACOS: {
        "brew": ["openssl", "libffi"],
    },
    OperatingSystem.LINUX: {
        "apt": ["python3-dev", "libssl-dev", "libffi-dev", "build-essential"],
        "yum": ["python3-devel", "openssl-devel", "libffi-devel", "gcc"],
        "dnf": ["python3-devel", "openssl-devel", "libffi-devel", "gcc"],
        "pacman": ["python", "openssl", "libffi", "base-devel"],
    },
    OperatingSystem.WINDOWS: {},  # Windows doesn't need system packages
}


@dataclass
class PackageStatus:
    """Status of a package installation."""

    name: str
    required_version: str
    installed_version: str | None
    is_installed: bool
    needs_upgrade: bool
    error: str | None = None


@dataclass
class DependencyCheckResult:
    """Result of dependency check."""

    packages: list[PackageStatus]
    missing_count: int
    upgrade_count: int
    all_satisfied: bool
    errors: list[str]


def parse_version(version_str: str) -> tuple[int, ...]:
    """
    Parse version string to comparable tuple.

    Args:
        version_str: Version string like "1.2.3".

    Returns:
        Tuple of version numbers.
    """
    try:
        # Handle versions like "1.2.3.post1" or "1.2.3a1"
        clean = version_str.split("+")[0].split(".post")[0].split("a")[0].split("b")[0]
        parts = clean.split(".")
        return tuple(int(p) for p in parts if p.isdigit())
    except (ValueError, AttributeError):
        return (0,)


def compare_versions(installed: str, required: str) -> int:
    """
    Compare two version strings.

    Args:
        installed: Installed version string.
        required: Required version string.

    Returns:
        -1 if installed < required, 0 if equal, 1 if installed > required.
    """
    inst_tuple = parse_version(installed)
    req_tuple = parse_version(required)

    # Pad shorter tuple with zeros
    max_len = max(len(inst_tuple), len(req_tuple))
    inst_tuple = inst_tuple + (0,) * (max_len - len(inst_tuple))
    req_tuple = req_tuple + (0,) * (max_len - len(req_tuple))

    if inst_tuple < req_tuple:
        return -1
    elif inst_tuple > req_tuple:
        return 1
    return 0


def get_installed_packages() -> dict[str, str]:
    """
    Get all installed Python packages and their versions.

    Returns:
        Dictionary of package_name -> version.
    """
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "list", "--format=json"],
            capture_output=True,
            text=True,
            timeout=60,
        )
        if result.returncode == 0:
            packages = json.loads(result.stdout)
            return {
                pkg["name"].lower().replace("_", "-"): pkg["version"]
                for pkg in packages
            }
    except Exception:
        pass

    # Fallback: try pip freeze
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "freeze"],
            capture_output=True,
            text=True,
            timeout=60,
        )
        if result.returncode == 0:
            packages = {}
            for line in result.stdout.strip().split("\n"):
                if "==" in line:
                    name, version = line.split("==", 1)
                    packages[name.lower().replace("_", "-")] = version
            return packages
    except Exception:
        pass

    return {}


def check_package_status(
    name: str,
    required_version: str,
    installed_packages: dict[str, str],
) -> PackageStatus:
    """
    Check the installation status of a single package.

    Args:
        name: Package name.
        required_version: Minimum required version.
        installed_packages: Dictionary of installed packages.

    Returns:
        PackageStatus for the package.
    """
    # Normalize name
    normalized_name = name.lower().replace("_", "-")
    installed_version = installed_packages.get(normalized_name)

    if installed_version is None:
        return PackageStatus(
            name=name,
            required_version=required_version,
            installed_version=None,
            is_installed=False,
            needs_upgrade=False,
        )

    needs_upgrade = compare_versions(installed_version, required_version) < 0

    return PackageStatus(
        name=name,
        required_version=required_version,
        installed_version=installed_version,
        is_installed=True,
        needs_upgrade=needs_upgrade,
    )


def check_dependencies(
    include_optional: bool = False,
) -> DependencyCheckResult:
    """
    Check all required dependencies.

    Args:
        include_optional: Whether to include optional packages.

    Returns:
        DependencyCheckResult with status of all packages.
    """
    installed = get_installed_packages()
    packages_to_check = dict(REQUIRED_PACKAGES)

    if include_optional:
        packages_to_check.update(OPTIONAL_PACKAGES)

    statuses: list[PackageStatus] = []
    errors: list[str] = []

    for name, version in packages_to_check.items():
        status = check_package_status(name, version, installed)
        statuses.append(status)

    missing_count = sum(1 for s in statuses if not s.is_installed)
    upgrade_count = sum(1 for s in statuses if s.needs_upgrade)

    return DependencyCheckResult(
        packages=statuses,
        missing_count=missing_count,
        upgrade_count=upgrade_count,
        all_satisfied=missing_count == 0 and upgrade_count == 0,
        errors=errors,
    )


def detect_package_manager(platform_info: PlatformInfo) -> str | None:
    """
    Detect the available package manager on Unix systems.

    Args:
        platform_info: Platform information.

    Returns:
        Package manager name or None.
    """
    if platform_info.is_windows:
        return None

    if platform_info.is_macos:
        # Check for Homebrew
        result = run_shell_command("which brew", platform_info, capture_output=True)
        if result.returncode == 0:
            return "brew"
        return None

    # Linux - check for various package managers
    managers = ["apt", "apt-get", "yum", "dnf", "pacman", "zypper"]
    for manager in managers:
        result = run_shell_command(f"which {manager}", platform_info, capture_output=True)
        if result.returncode == 0:
            # Normalize apt-get to apt
            return "apt" if manager == "apt-get" else manager

    return None


def install_system_dependencies(
    platform_info: PlatformInfo,
    progress_callback: Callable[[str, int], None] | None = None,
) -> tuple[bool, list[str]]:
    """
    Install system-level dependencies.

    Args:
        platform_info: Platform information.
        progress_callback: Optional callback for progress updates.

    Returns:
        Tuple of (success, list of errors).
    """
    errors: list[str] = []

    if platform_info.is_windows:
        # Windows doesn't typically need system packages
        return True, errors

    package_manager = detect_package_manager(platform_info)
    if package_manager is None:
        return True, errors  # No package manager, skip system deps

    deps = SYSTEM_DEPENDENCIES.get(platform_info.os_type, {})
    packages = deps.get(package_manager, [])

    if not packages:
        return True, errors

    if progress_callback:
        progress_callback("Installing system dependencies...", 0)

    # Build install command
    if package_manager == "brew":
        cmd = f"brew install {' '.join(packages)}"
        use_sudo = False
    elif package_manager in ("apt", "apt-get"):
        cmd = f"apt-get update && apt-get install -y {' '.join(packages)}"
        use_sudo = not platform_info.is_admin
    elif package_manager in ("yum", "dnf"):
        cmd = f"{package_manager} install -y {' '.join(packages)}"
        use_sudo = not platform_info.is_admin
    elif package_manager == "pacman":
        cmd = f"pacman -S --noconfirm {' '.join(packages)}"
        use_sudo = not platform_info.is_admin
    else:
        return True, errors

    if use_sudo and platform_info.has_sudo:
        cmd = f"sudo {cmd}"
    elif use_sudo and not platform_info.has_sudo:
        errors.append("System dependencies require root privileges. Please run with sudo.")
        return False, errors

    try:
        result = run_shell_command(cmd, platform_info, capture_output=True, timeout=300)
        if result.returncode != 0:
            errors.append(f"Failed to install system dependencies: {result.stderr}")
            return False, errors
    except subprocess.TimeoutExpired:
        errors.append("System dependency installation timed out")
        return False, errors
    except Exception as e:
        errors.append(f"Error installing system dependencies: {e}")
        return False, errors

    if progress_callback:
        progress_callback("System dependencies installed", 100)

    return True, errors


def install_python_packages(
    packages: list[PackageStatus],
    platform_info: PlatformInfo,
    progress_callback: Callable[[str, int], None] | None = None,
    upgrade_pip: bool = True,
) -> tuple[bool, list[str]]:
    """
    Install or upgrade Python packages.

    Args:
        packages: List of packages to install/upgrade.
        platform_info: Platform information.
        progress_callback: Optional callback for progress updates.
        upgrade_pip: Whether to upgrade pip first.

    Returns:
        Tuple of (success, list of errors).
    """
    errors: list[str] = []
    total = len(packages) + (1 if upgrade_pip else 0)
    current = 0

    # Upgrade pip first
    if upgrade_pip:
        if progress_callback:
            progress_callback("Upgrading pip...", int(current / total * 100))

        try:
            result = subprocess.run(
                [sys.executable, "-m", "pip", "install", "--upgrade", "pip"],
                capture_output=True,
                text=True,
                timeout=120,
            )
            if result.returncode != 0:
                errors.append(f"Warning: pip upgrade failed: {result.stderr}")
        except Exception as e:
            errors.append(f"Warning: pip upgrade failed: {e}")

        current += 1

    # Install/upgrade packages
    for pkg in packages:
        if progress_callback:
            action = "Upgrading" if pkg.needs_upgrade else "Installing"
            progress_callback(f"{action} {pkg.name}...", int(current / total * 100))

        package_spec = f"{pkg.name}>={pkg.required_version}"

        try:
            cmd = [sys.executable, "-m", "pip", "install", "--upgrade", package_spec]
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                timeout=180,
            )
            if result.returncode != 0:
                errors.append(f"Failed to install {pkg.name}: {result.stderr}")
        except subprocess.TimeoutExpired:
            errors.append(f"Installation of {pkg.name} timed out")
        except Exception as e:
            errors.append(f"Error installing {pkg.name}: {e}")

        current += 1

    if progress_callback:
        progress_callback("Package installation complete", 100)

    return len(errors) == 0, errors


def install_all_dependencies(
    platform_info: PlatformInfo,
    progress_callback: Callable[[str, int], None] | None = None,
    include_optional: bool = False,
) -> tuple[bool, list[str]]:
    """
    Install all required dependencies.

    Args:
        platform_info: Platform information.
        progress_callback: Optional callback for progress updates.
        include_optional: Whether to include optional packages.

    Returns:
        Tuple of (success, list of errors).
    """
    all_errors: list[str] = []

    # Phase 1: System dependencies
    if progress_callback:
        progress_callback("Checking system dependencies...", 0)

    success, errors = install_system_dependencies(platform_info, progress_callback)
    all_errors.extend(errors)

    # Phase 2: Check Python packages
    if progress_callback:
        progress_callback("Checking Python packages...", 25)

    check_result = check_dependencies(include_optional)

    # Get packages that need installation or upgrade
    packages_to_install = [
        pkg for pkg in check_result.packages
        if not pkg.is_installed or pkg.needs_upgrade
    ]

    if not packages_to_install:
        if progress_callback:
            progress_callback("All dependencies satisfied", 100)
        return True, all_errors

    # Phase 3: Install Python packages
    def wrapped_callback(msg: str, pct: int) -> None:
        if progress_callback:
            # Scale 0-100 to 25-100
            scaled_pct = 25 + int(pct * 0.75)
            progress_callback(msg, scaled_pct)

    success, errors = install_python_packages(
        packages_to_install,
        platform_info,
        wrapped_callback,
    )
    all_errors.extend(errors)

    return len(all_errors) == 0, all_errors


def verify_installation() -> tuple[bool, str]:
    """
    Verify that all dependencies are properly installed.

    Returns:
        Tuple of (success, message).
    """
    result = check_dependencies()

    if result.all_satisfied:
        return True, "All dependencies verified successfully"

    messages = []
    for pkg in result.packages:
        if not pkg.is_installed:
            messages.append(f"Missing: {pkg.name} (>={pkg.required_version})")
        elif pkg.needs_upgrade:
            messages.append(
                f"Needs upgrade: {pkg.name} "
                f"({pkg.installed_version} -> >={pkg.required_version})"
            )

    return False, "\n".join(messages)



"""
Agent Orchestrator for AiCippy.

Coordinates multiple specialized agents to complete complex tasks
with parallel execution, dependency resolution, and result merging.

This module provides:
- Multi-agent task orchestration
- Parallel execution with concurrency limits
- Task decomposition and dependency resolution
- Bedrock model invocation with circuit breaker pattern
- Progress tracking and result aggregation
"""

from __future__ import annotations

import asyncio
import json
import uuid
from collections.abc import Callable, Sequence
from dataclasses import dataclass, field
from datetime import datetime, timezone
from typing import TYPE_CHECKING, Any, Final

import boto3
from botocore.config import Config
from botocore.exceptions import BotoCoreError, ClientError
from rich.console import Console
from rich.progress import BarColumn, Progress, SpinnerColumn, TaskID, TextColumn

from aicippy.agents.models import (
    AGENT_SYSTEM_PROMPTS,
    AgentConfig,
    AgentResponse,
    AgentRun,
    AgentStatus,
    AgentTask,
    AgentType,
    TokenUsage,
)
from aicippy.config import get_settings
from aicippy.exceptions import (
    AgentExecutionError,
    AgentSpawnError,
    AgentTimeoutError,
    ContextExceededError,
    ErrorContext,
    ModelInvocationError,
    ModelRateLimitedError,
)
from aicippy.types import JsonDict, SessionId
from aicippy.utils.correlation import CorrelationContext, get_correlation_id
from aicippy.utils.logging import get_logger
from aicippy.utils.retry import CircuitBreaker, CircuitBreakerOpenError, async_retry

if TYPE_CHECKING:
    from types import TracebackType

logger = get_logger(__name__)

# ============================================================================
# Constants
# ============================================================================

DEFAULT_MAX_AGENTS: Final[int] = 3
MAX_TASK_DECOMPOSITION_DEPTH: Final[int] = 3
BEDROCK_READ_TIMEOUT: Final[int] = 120
BEDROCK_CONNECT_TIMEOUT: Final[int] = 10
BEDROCK_MAX_RETRIES: Final[int] = 3


# ============================================================================
# Data Classes
# ============================================================================


@dataclass(slots=True)
class AgentInstance:
    """
    Running agent instance with state tracking.

    Attributes:
        id: Unique agent identifier.
        agent_type: Type of specialized agent.
        config: Agent configuration.
        status: Current execution status.
        current_task: Currently executing task.
        total_tokens: Cumulative token usage.
        started_at: Instance creation timestamp.
        last_activity: Last activity timestamp.
    """

    id: str
    agent_type: AgentType
    config: AgentConfig
    status: AgentStatus = AgentStatus.IDLE
    current_task: AgentTask | None = None
    total_tokens: TokenUsage = field(default_factory=TokenUsage)
    started_at: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    last_activity: datetime = field(default_factory=lambda: datetime.now(timezone.utc))

    def update_activity(self) -> None:
        """Update last activity timestamp."""
        self.last_activity = datetime.now(timezone.utc)

    def add_tokens(self, usage: TokenUsage) -> None:
        """Add token usage to cumulative total."""
        self.total_tokens = self.total_tokens + usage
        self.update_activity()


# ============================================================================
# Orchestrator
# ============================================================================


class AgentOrchestrator:
    """
    Multi-agent orchestrator for complex task execution.

    Coordinates specialized agents, manages parallel execution,
    handles dependencies, and merges results with comprehensive
    error handling and observability.

    Features:
    - Parallel agent execution up to configurable limit
    - Task decomposition with dependency resolution
    - Circuit breaker pattern for Bedrock API
    - Structured logging with correlation IDs
    - Graceful error recovery

    Example:
        >>> orchestrator = AgentOrchestrator(model_id="opus", max_agents=5)
        >>> response = await orchestrator.chat("Explain AWS Lambda")
        >>> print(response.content)

    Example with progress:
        >>> from rich.console import Console
        >>> console = Console()
        >>> response = await orchestrator.run_task_with_progress(
        ...     "Deploy a serverless API",
        ...     console,
        ... )
    """

    __slots__ = (
        "_settings",
        "_model_id",
        "_max_agents",
        "_session_id",
        "_agents",
        "_task_queue",
        "_completed_tasks",
        "_lock",
        "_circuit_breaker",
        "_bedrock_client",
    )

    def __init__(
        self,
        model_id: str | None = None,
        max_agents: int = DEFAULT_MAX_AGENTS,
        session_id: str | None = None,
    ) -> None:
        """
        Initialize the orchestrator.

        Args:
            model_id: Bedrock model ID to use. Defaults to configured default.
            max_agents: Maximum number of parallel agents (capped by settings).
            session_id: Session identifier for tracking. Auto-generated if not provided.

        Raises:
            ValueError: If max_agents is less than 1.
        """
        if max_agents < 1:
            raise ValueError("max_agents must be at least 1")

        self._settings = get_settings()
        self._model_id = model_id or self._settings.get_model_id()
        self._max_agents = min(max_agents, self._settings.max_parallel_agents)
        self._session_id = SessionId(session_id or str(uuid.uuid4()))

        # Agent management
        self._agents: dict[str, AgentInstance] = {}
        self._task_queue: asyncio.Queue[AgentTask] = asyncio.Queue()
        self._completed_tasks: dict[str, AgentResponse] = {}
        self._lock = asyncio.Lock()

        # Circuit breaker for Bedrock API
        self._circuit_breaker = CircuitBreaker(
            failure_threshold=5,
            recovery_timeout=30.0,
            half_open_requests=3,
        )

        # Bedrock client (lazy initialization)
        self._bedrock_client: Any = None

        logger.info(
            "orchestrator_initialized",
            model_id=self._model_id,
            max_agents=self._max_agents,
            session_id=self._session_id,
        )

    @property
    def session_id(self) -> SessionId:
        """Get the session identifier."""
        return self._session_id

    @property
    def model_id(self) -> str:
        """Get the current model ID."""
        return self._model_id

    @property
    def max_agents(self) -> int:
        """Get the maximum number of parallel agents."""
        return self._max_agents

    @property
    def active_agent_count(self) -> int:
        """Get the number of currently active agents."""
        return len(self._agents)

    def _get_bedrock_client(self) -> Any:
        """
        Get or create Bedrock runtime client with optimized configuration.

        Returns:
            Configured boto3 Bedrock runtime client.
        """
        if self._bedrock_client is None:
            config = Config(
                retries={
                    "max_attempts": BEDROCK_MAX_RETRIES,
                    "mode": "adaptive",
                },
                read_timeout=BEDROCK_READ_TIMEOUT,
                connect_timeout=BEDROCK_CONNECT_TIMEOUT,
            )

            session_kwargs: dict[str, Any] = {
                "region_name": self._settings.aws_region,
            }
            if self._settings.aws_profile:
                session_kwargs["profile_name"] = self._settings.aws_profile

            session = boto3.Session(**session_kwargs)
            self._bedrock_client = session.client(
                "bedrock-runtime",
                config=config,
            )

        return self._bedrock_client

    async def chat(self, message: str) -> AgentResponse:
        """
        Process a chat message and return response.

        Args:
            message: User message to process.

        Returns:
            AgentResponse with the assistant's reply.

        Note:
            This method handles errors gracefully and returns an error
            response rather than raising exceptions for operational failures.
        """
        if not message or not message.strip():
            return AgentResponse.error_response(
                "Empty message provided",
                AgentType.ORCHESTRATOR,
            )

        correlation_id = get_correlation_id()
        logger.info(
            "chat_started",
            message_length=len(message),
            model=self._model_id,
            correlation_id=correlation_id,
        )

        try:
            response = await self._invoke_model(
                message=message,
                system_prompt=AGENT_SYSTEM_PROMPTS[AgentType.ORCHESTRATOR],
            )

            logger.info(
                "chat_completed",
                input_tokens=response.usage.input_tokens if response.usage else 0,
                output_tokens=response.usage.output_tokens if response.usage else 0,
                correlation_id=correlation_id,
            )

            return response

        except CircuitBreakerOpenError as e:
            logger.warning(
                "chat_circuit_breaker_open",
                error=str(e),
                correlation_id=correlation_id,
            )
            return AgentResponse.error_response(
                "Service temporarily unavailable. Please retry shortly.",
                AgentType.ORCHESTRATOR,
            )

        except (ModelInvocationError, ModelRateLimitedError) as e:
            logger.warning(
                "chat_model_error",
                error=str(e),
                error_code=e.code.name,
                correlation_id=correlation_id,
            )
            return AgentResponse.error_response(str(e), AgentType.ORCHESTRATOR)

        except Exception as e:
            logger.exception(
                "chat_failed",
                error=str(e),
                correlation_id=correlation_id,
            )
            return AgentResponse.error_response(
                f"Unexpected error: {e}",
                AgentType.ORCHESTRATOR,
            )

    async def run_task_with_progress(
        self,
        task: str,
        console: Console,
    ) -> AgentResponse:
        """
        Execute a task with progress display.

        Args:
            task: Task description to execute.
            console: Rich console for progress display.

        Returns:
            AgentResponse with combined results from all agents.

        Note:
            Complex tasks are automatically decomposed into subtasks
            and executed in parallel where dependencies allow.
        """
        if not task or not task.strip():
            return AgentResponse.error_response(
                "Empty task provided",
                AgentType.ORCHESTRATOR,
            )

        correlation_id = get_correlation_id()
        logger.info(
            "task_started",
            task=task[:100],
            max_agents=self._max_agents,
            correlation_id=correlation_id,
        )

        try:
            # Analyze task and create subtasks
            subtasks = await self._analyze_and_decompose(task)

            if not subtasks:
                # Simple task - process directly
                logger.debug(
                    "task_simple_execution",
                    correlation_id=correlation_id,
                )
                return await self.chat(task)

            logger.info(
                "task_decomposed",
                subtask_count=len(subtasks),
                correlation_id=correlation_id,
            )

            # Create progress display
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                BarColumn(),
                TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
                console=console,
                transient=True,
            ) as progress:
                # Add progress tasks for each subtask
                progress_tasks: dict[str, TaskID] = {}
                for subtask in subtasks:
                    task_id = progress.add_task(
                        f"[cyan]{subtask.agent_type.value}[/cyan]: {subtask.description[:50]}",
                        total=100,
                    )
                    progress_tasks[subtask.id] = task_id

                # Execute subtasks with parallelism
                results = await self._execute_subtasks(
                    subtasks,
                    lambda task_id, pct: progress.update(
                        progress_tasks.get(task_id, TaskID(0)),
                        completed=pct,
                    ),
                )

            # Merge results
            merged = self._merge_results(results)

            logger.info(
                "task_completed",
                subtask_count=len(subtasks),
                successful=sum(1 for r in results.values() if not r.is_error),
                failed=sum(1 for r in results.values() if r.is_error),
                correlation_id=correlation_id,
            )

            return merged

        except Exception as e:
            logger.exception(
                "task_failed",
                error=str(e),
                correlation_id=correlation_id,
            )
            return AgentResponse.error_response(
                f"Task execution failed: {e}",
                AgentType.ORCHESTRATOR,
            )

    async def _analyze_and_decompose(self, task: str) -> list[AgentTask]:
        """
        Analyze a task and decompose into subtasks for parallel execution.

        Args:
            task: Task description to analyze.

        Returns:
            List of subtasks. Empty list if task is simple.
        """
        analysis_prompt = f"""Analyze this task and determine if it should be decomposed into subtasks
for parallel agent execution. If the task is simple, return an empty list.
If complex, identify the subtasks and which specialized agent should handle each.

Available agent types:
- infra-core: AWS CDK, IAM, VPC, security
- bedrock-ai: Bedrock agents, knowledge bases
- api-gateway: WebSocket, Lambda authorizers
- cognito-auth: User pools, authentication
- cli-core: Python CLI, Typer, Rich
- mcp-bridges: Tool connectors
- knowledge-ingest: Feed crawling, indexing
- observability: CloudWatch, logging
- email-notify: SES, notifications
- cicd-deploy: GitHub Actions, deployment

Task: {task}

Respond with a JSON array of subtasks, each with:
- description: Brief task description
- agent_type: One of the agent types above
- priority: 0-10 (higher = more important)
- dependencies: Array of other subtask indices that must complete first

Example response for a complex task:
[
  {{"description": "Create VPC and networking", "agent_type": "infra-core", "priority": 10, "dependencies": []}},
  {{"description": "Setup DynamoDB tables", "agent_type": "infra-core", "priority": 8, "dependencies": [0]}}
]

For a simple task, respond with: []
"""

        try:
            response = await self._invoke_model(
                message=analysis_prompt,
                system_prompt="You are a task analysis assistant. Respond only with valid JSON.",
            )

            # Parse JSON response
            content = response.content.strip()

            # Extract JSON from markdown code blocks if present
            if "```json" in content:
                content = content.split("```json")[1].split("```")[0]
            elif "```" in content:
                content = content.split("```")[1].split("```")[0]

            content = content.strip()

            try:
                subtask_data = json.loads(content)
            except json.JSONDecodeError as e:
                logger.warning(
                    "task_decomposition_json_error",
                    error=str(e),
                    content_preview=content[:200],
                )
                return []

            if not subtask_data or not isinstance(subtask_data, list):
                return []

            # Convert to AgentTask objects with validation
            subtasks: list[AgentTask] = []
            for i, data in enumerate(subtask_data):
                if not isinstance(data, dict):
                    continue

                description = data.get("description", "")
                agent_type_str = data.get("agent_type", "")

                if not description or not agent_type_str:
                    continue

                try:
                    agent_type = AgentType.from_string(agent_type_str)
                except (ValueError, KeyError):
                    logger.warning(
                        "task_decomposition_invalid_agent_type",
                        agent_type=agent_type_str,
                    )
                    continue

                task_id = str(uuid.uuid4())
                dependencies = []

                # Resolve dependency indices to task IDs
                for dep_idx in data.get("dependencies", []):
                    if isinstance(dep_idx, int) and 0 <= dep_idx < len(subtasks):
                        dependencies.append(subtasks[dep_idx].id)

                subtasks.append(
                    AgentTask(
                        id=task_id,
                        description=description,
                        agent_type=agent_type,
                        priority=min(10, max(0, int(data.get("priority", 5)))),
                        dependencies=dependencies,
                    )
                )

            return subtasks

        except Exception as e:
            logger.warning(
                "task_decomposition_failed",
                error=str(e),
            )
            return []

    async def _execute_subtasks(
        self,
        subtasks: Sequence[AgentTask],
        progress_callback: Callable[[str, int], None],
    ) -> dict[str, AgentResponse]:
        """
        Execute subtasks with parallel execution and dependency handling.

        Args:
            subtasks: List of subtasks to execute.
            progress_callback: Callback for progress updates (task_id, percentage).

        Returns:
            Dictionary mapping task IDs to responses.
        """
        results: dict[str, AgentResponse] = {}
        pending: dict[str, AgentTask] = {task.id: task for task in subtasks}
        running: dict[str, asyncio.Task[AgentResponse]] = {}

        while pending or running:
            # Find tasks that can be started (all dependencies completed)
            ready = [
                task
                for task in pending.values()
                if all(dep in results for dep in task.dependencies)
            ]

            # Sort by priority (higher first) and start tasks up to max_agents
            ready.sort(key=lambda t: t.priority, reverse=True)

            for task in ready:
                if len(running) >= self._max_agents:
                    break

                # Start task execution
                del pending[task.id]
                progress_callback(task.id, 10)

                async_task = asyncio.create_task(
                    self._execute_single_task(task, results, progress_callback),
                    name=f"agent-{task.agent_type.value}-{task.id[:8]}",
                )
                running[task.id] = async_task

            if running:
                # Wait for at least one task to complete
                done, _ = await asyncio.wait(
                    running.values(),
                    return_when=asyncio.FIRST_COMPLETED,
                )

                # Process completed tasks
                for completed_task in done:
                    # Find which task completed
                    completed_id: str | None = None
                    for tid, t in running.items():
                        if t is completed_task:
                            completed_id = tid
                            break

                    if completed_id:
                        try:
                            results[completed_id] = completed_task.result()
                        except Exception as e:
                            logger.warning(
                                "subtask_exception",
                                task_id=completed_id,
                                error=str(e),
                            )
                            results[completed_id] = AgentResponse.error_response(
                                str(e),
                                AgentType.ORCHESTRATOR,
                            )

                        del running[completed_id]
                        progress_callback(completed_id, 100)

            # Small delay to prevent tight loop
            await asyncio.sleep(0.05)

        return results

    async def _execute_single_task(
        self,
        task: AgentTask,
        completed_results: dict[str, AgentResponse],
        progress_callback: Callable[[str, int], None],
    ) -> AgentResponse:
        """
        Execute a single subtask with context from dependencies.

        Args:
            task: Task to execute.
            completed_results: Results from completed dependent tasks.
            progress_callback: Progress update callback.

        Returns:
            AgentResponse from task execution.
        """
        correlation_id = get_correlation_id()
        progress_callback(task.id, 20)

        # Build context from dependencies
        context_parts: list[str] = []
        for dep_id in task.dependencies:
            if dep_id in completed_results:
                dep_result = completed_results[dep_id]
                if not dep_result.is_error:
                    # Truncate long results to prevent context overflow
                    content_preview = dep_result.content[:1000]
                    if len(dep_result.content) > 1000:
                        content_preview += "... [truncated]"
                    context_parts.append(f"Previous result: {content_preview}")

        context = "\n\n".join(context_parts) if context_parts else ""
        progress_callback(task.id, 40)

        # Get system prompt for this agent type
        system_prompt = AGENT_SYSTEM_PROMPTS.get(
            task.agent_type,
            AGENT_SYSTEM_PROMPTS[AgentType.ORCHESTRATOR],
        )

        # Build message with context
        message_parts = [f"Execute this task:\n{task.description}"]
        if context:
            message_parts.append(f"\nContext from previous tasks:\n{context}")
        message_parts.append(
            "\nProvide a detailed response with your analysis and any code, "
            "configurations, or recommendations needed to complete this task."
        )

        message = "\n".join(message_parts)
        progress_callback(task.id, 60)

        # Execute with the specialized agent
        try:
            response = await self._invoke_model(
                message=message,
                system_prompt=system_prompt,
            )

            response.agent_type = task.agent_type
            response.task_id = task.id

            progress_callback(task.id, 90)

            logger.debug(
                "subtask_completed",
                task_id=task.id,
                agent_type=task.agent_type.value,
                correlation_id=correlation_id,
            )

            return response

        except Exception as e:
            logger.warning(
                "subtask_failed",
                task_id=task.id,
                agent_type=task.agent_type.value,
                error=str(e),
                correlation_id=correlation_id,
            )
            return AgentResponse.error_response(str(e), task.agent_type)

    def _merge_results(
        self,
        results: dict[str, AgentResponse],
    ) -> AgentResponse:
        """
        Merge results from multiple agents into a single response.

        Args:
            results: Dictionary of task results.

        Returns:
            Merged AgentResponse with combined content and usage.
        """
        if not results:
            return AgentResponse(content="No results to merge.")

        # Combine content by agent type
        content_parts: list[str] = []
        total_usage = TokenUsage()
        all_artifacts: list[dict[str, Any]] = []
        error_count = 0

        for task_id, response in results.items():
            agent_label = response.agent_type.value if response.agent_type else "unknown"

            if response.is_error:
                error_count += 1
                content_parts.append(f"## {agent_label} (Error)\n{response.error}")
            else:
                content_parts.append(f"## {agent_label}\n{response.content}")

            if response.usage:
                total_usage = total_usage + response.usage

            all_artifacts.extend(response.artifacts)

        merged_content = "\n\n---\n\n".join(content_parts)

        # Add summary header if there were errors
        if error_count > 0:
            header = f"**Note:** {error_count} of {len(results)} tasks encountered errors.\n\n"
            merged_content = header + merged_content

        return AgentResponse(
            content=merged_content,
            agent_type=AgentType.ORCHESTRATOR,
            usage=total_usage,
            artifacts=all_artifacts,
        )

    @async_retry(max_attempts=3, min_wait=1.0, max_wait=10.0)
    async def _invoke_model(
        self,
        message: str,
        system_prompt: str,
    ) -> AgentResponse:
        """
        Invoke Bedrock model with retry logic and circuit breaker.

        Args:
            message: User message to send.
            system_prompt: System prompt for the model.

        Returns:
            AgentResponse with model output.

        Raises:
            ModelInvocationError: If model invocation fails.
            ModelRateLimitedError: If rate limited.
            CircuitBreakerOpenError: If circuit breaker is open.
        """
        correlation_id = get_correlation_id()
        client = self._get_bedrock_client()

        # Build request body based on model
        body: dict[str, Any]
        if "claude" in self._model_id.lower():
            body = {
                "anthropic_version": "bedrock-2023-05-31",
                "max_tokens": 4096,
                "system": system_prompt,
                "messages": [{"role": "user", "content": message}],
            }
        elif "llama" in self._model_id.lower():
            body = {
                "prompt": f"<|system|>\n{system_prompt}<|end|>\n<|user|>\n{message}<|end|>\n<|assistant|>",
                "max_gen_len": 4096,
                "temperature": 0.7,
            }
        else:
            # Default format for other models
            body = {
                "prompt": f"{system_prompt}\n\nHuman: {message}\n\nAssistant:",
                "max_tokens": 4096,
            }

        # Async invoke function for circuit breaker
        async def invoke() -> dict[str, Any]:
            loop = asyncio.get_running_loop()
            try:
                response = await loop.run_in_executor(
                    None,
                    lambda: client.invoke_model(
                        modelId=self._model_id,
                        body=json.dumps(body),
                        contentType="application/json",
                        accept="application/json",
                    ),
                )
                return json.loads(response["body"].read())
            except ClientError as e:
                error_code = e.response.get("Error", {}).get("Code", "")
                error_message = e.response.get("Error", {}).get("Message", str(e))

                if error_code == "ThrottlingException":
                    raise ModelRateLimitedError(
                        model_id=self._model_id,
                        context=ErrorContext(
                            correlation_id=correlation_id,
                            operation="model_invocation",
                        ),
                    ) from e
                elif error_code == "ValidationException" and "context" in error_message.lower():
                    raise ContextExceededError(
                        model_id=self._model_id,
                        tokens_used=0,  # Unknown
                        max_tokens=200000,
                        context=ErrorContext(
                            correlation_id=correlation_id,
                            operation="model_invocation",
                        ),
                    ) from e
                else:
                    raise ModelInvocationError(
                        model_id=self._model_id,
                        message=error_message,
                        context=ErrorContext(
                            correlation_id=correlation_id,
                            operation="model_invocation",
                            details={"error_code": error_code},
                        ),
                    ) from e
            except BotoCoreError as e:
                raise ModelInvocationError(
                    model_id=self._model_id,
                    message=str(e),
                    context=ErrorContext(
                        correlation_id=correlation_id,
                        operation="model_invocation",
                    ),
                ) from e

        # Execute through circuit breaker
        try:
            result = await self._circuit_breaker.call(invoke)
        except CircuitBreakerOpenError:
            logger.warning(
                "circuit_breaker_open",
                model=self._model_id,
                correlation_id=correlation_id,
            )
            raise
        except (ModelInvocationError, ModelRateLimitedError, ContextExceededError):
            raise
        except Exception as e:
            logger.error(
                "model_invocation_failed",
                error=str(e),
                model=self._model_id,
                correlation_id=correlation_id,
            )
            raise ModelInvocationError(
                model_id=self._model_id,
                message=str(e),
                context=ErrorContext(
                    correlation_id=correlation_id,
                    operation="model_invocation",
                ),
            ) from e

        # Parse response based on model
        if "claude" in self._model_id.lower():
            content = result.get("content", [{}])[0].get("text", "")
            usage = TokenUsage(
                input_tokens=result.get("usage", {}).get("input_tokens", 0),
                output_tokens=result.get("usage", {}).get("output_tokens", 0),
            )
        elif "llama" in self._model_id.lower():
            content = result.get("generation", "")
            usage = TokenUsage(
                input_tokens=result.get("prompt_token_count", 0),
                output_tokens=result.get("generation_token_count", 0),
            )
        else:
            content = result.get("completion", result.get("generated_text", ""))
            usage = TokenUsage()

        return AgentResponse(
            content=content,
            usage=usage,
        )

    async def close(self) -> None:
        """Clean up orchestrator resources."""
        self._agents.clear()
        self._completed_tasks.clear()

        # Clear task queue
        while not self._task_queue.empty():
            try:
                self._task_queue.get_nowait()
            except asyncio.QueueEmpty:
                break

        logger.debug("orchestrator_closed", session_id=self._session_id)

    async def __aenter__(self) -> "AgentOrchestrator":
        """Async context manager entry."""
        return self

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        """Async context manager exit with cleanup."""
        await self.close()



"""
Session Manager for AiCippy.

Handles user authentication sessions with automatic timeout after 12 hours
of inactivity or usage. Supports secure token storage and validation.
"""

from __future__ import annotations

import hashlib
import json
import os
import secrets
import time
from dataclasses import dataclass, field
from datetime import datetime, timedelta, timezone
from pathlib import Path
from typing import Final

from aicippy.installer.platform_detect import PlatformInfo, detect_platform

# Session configuration
SESSION_TIMEOUT_HOURS: Final[int] = 12
SESSION_FILE_NAME: Final[str] = "session.json"
TOKEN_LENGTH: Final[int] = 32

# Session states
SESSION_VALID: Final[str] = "valid"
SESSION_EXPIRED: Final[str] = "expired"
SESSION_INVALID: Final[str] = "invalid"
SESSION_MISSING: Final[str] = "missing"


@dataclass
class SessionInfo:
    """User session information."""

    user_id: str
    username: str
    email: str
    session_token: str
    created_at: datetime
    last_activity: datetime
    expires_at: datetime
    device_id: str
    ip_address: str | None = None
    metadata: dict[str, str] = field(default_factory=dict)

    def to_dict(self) -> dict:
        """Convert session to dictionary for storage."""
        return {
            "user_id": self.user_id,
            "username": self.username,
            "email": self.email,
            "session_token": self.session_token,
            "created_at": self.created_at.isoformat(),
            "last_activity": self.last_activity.isoformat(),
            "expires_at": self.expires_at.isoformat(),
            "device_id": self.device_id,
            "ip_address": self.ip_address,
            "metadata": self.metadata,
        }

    @classmethod
    def from_dict(cls, data: dict) -> SessionInfo:
        """Create session from dictionary."""
        return cls(
            user_id=data["user_id"],
            username=data["username"],
            email=data["email"],
            session_token=data["session_token"],
            created_at=datetime.fromisoformat(data["created_at"]),
            last_activity=datetime.fromisoformat(data["last_activity"]),
            expires_at=datetime.fromisoformat(data["expires_at"]),
            device_id=data["device_id"],
            ip_address=data.get("ip_address"),
            metadata=data.get("metadata", {}),
        )

    @property
    def is_expired(self) -> bool:
        """Check if session has expired."""
        now = datetime.now(timezone.utc)
        return now >= self.expires_at

    @property
    def time_remaining(self) -> timedelta:
        """Get time remaining until expiration."""
        now = datetime.now(timezone.utc)
        remaining = self.expires_at - now
        return remaining if remaining.total_seconds() > 0 else timedelta(0)

    @property
    def hours_remaining(self) -> float:
        """Get hours remaining until expiration."""
        return self.time_remaining.total_seconds() / 3600


@dataclass
class SessionValidationResult:
    """Result of session validation."""

    status: str
    session: SessionInfo | None
    message: str
    needs_login: bool


def generate_session_token() -> str:
    """Generate a secure random session token."""
    return secrets.token_hex(TOKEN_LENGTH)


def generate_device_id() -> str:
    """Generate a unique device identifier based on system info."""
    platform_info = detect_platform()

    # Combine system identifiers
    components = [
        platform_info.os_name,
        platform_info.os_version,
        str(platform_info.architecture),
        platform_info.home_dir.name,
        os.environ.get("USER", os.environ.get("USERNAME", "unknown")),
    ]

    # Create a hash
    combined = "|".join(components)
    return hashlib.sha256(combined.encode()).hexdigest()[:16]


def get_session_file_path(platform_info: PlatformInfo | None = None) -> Path:
    """
    Get the path to the session file.

    Args:
        platform_info: Optional platform info (will be detected if not provided).

    Returns:
        Path to the session file.
    """
    if platform_info is None:
        platform_info = detect_platform()

    sessions_dir = platform_info.config_path / "sessions"
    return sessions_dir / SESSION_FILE_NAME


def create_session(
    user_id: str,
    username: str,
    email: str,
    ip_address: str | None = None,
    metadata: dict[str, str] | None = None,
) -> SessionInfo:
    """
    Create a new user session.

    Args:
        user_id: Unique user identifier.
        username: User's display name.
        email: User's email address.
        ip_address: Optional IP address.
        metadata: Optional additional metadata.

    Returns:
        New SessionInfo instance.
    """
    now = datetime.now(timezone.utc)
    expires_at = now + timedelta(hours=SESSION_TIMEOUT_HOURS)

    return SessionInfo(
        user_id=user_id,
        username=username,
        email=email,
        session_token=generate_session_token(),
        created_at=now,
        last_activity=now,
        expires_at=expires_at,
        device_id=generate_device_id(),
        ip_address=ip_address,
        metadata=metadata or {},
    )


def save_session(
    session: SessionInfo,
    platform_info: PlatformInfo | None = None,
) -> tuple[bool, str]:
    """
    Save session to secure storage.

    Args:
        session: Session to save.
        platform_info: Optional platform info.

    Returns:
        Tuple of (success, message).
    """
    try:
        session_path = get_session_file_path(platform_info)

        # Ensure directory exists
        session_path.parent.mkdir(parents=True, exist_ok=True)

        # Write session data
        session_data = session.to_dict()
        session_path.write_text(json.dumps(session_data, indent=2))

        # Secure the file (Unix only)
        if os.name != "nt":
            os.chmod(session_path, 0o600)

        return True, f"Session saved for {session.username}"

    except PermissionError:
        return False, "Permission denied when saving session"
    except Exception as e:
        return False, f"Failed to save session: {e}"


def load_session(
    platform_info: PlatformInfo | None = None,
) -> SessionInfo | None:
    """
    Load session from storage.

    Args:
        platform_info: Optional platform info.

    Returns:
        SessionInfo if found, None otherwise.
    """
    try:
        session_path = get_session_file_path(platform_info)

        if not session_path.exists():
            return None

        session_data = json.loads(session_path.read_text())
        return SessionInfo.from_dict(session_data)

    except (json.JSONDecodeError, KeyError, ValueError):
        # Corrupted session file
        return None
    except Exception:
        return None


def validate_session(
    platform_info: PlatformInfo | None = None,
) -> SessionValidationResult:
    """
    Validate the current session.

    Args:
        platform_info: Optional platform info.

    Returns:
        SessionValidationResult with status and details.
    """
    session = load_session(platform_info)

    if session is None:
        return SessionValidationResult(
            status=SESSION_MISSING,
            session=None,
            message="No active session found. Please login.",
            needs_login=True,
        )

    # Check if session has expired
    if session.is_expired:
        return SessionValidationResult(
            status=SESSION_EXPIRED,
            session=session,
            message=f"Session expired. Last activity was {session.last_activity.strftime('%Y-%m-%d %H:%M:%S')} UTC.",
            needs_login=True,
        )

    # Check device ID matches
    current_device_id = generate_device_id()
    if session.device_id != current_device_id:
        return SessionValidationResult(
            status=SESSION_INVALID,
            session=session,
            message="Session was created on a different device. Please login again.",
            needs_login=True,
        )

    # Session is valid
    hours_left = session.hours_remaining
    return SessionValidationResult(
        status=SESSION_VALID,
        session=session,
        message=f"Welcome back, {session.username}! Session valid for {hours_left:.1f} more hours.",
        needs_login=False,
    )


def refresh_session(
    platform_info: PlatformInfo | None = None,
) -> tuple[bool, str]:
    """
    Refresh the current session's activity timestamp and extend expiration.

    Args:
        platform_info: Optional platform info.

    Returns:
        Tuple of (success, message).
    """
    session = load_session(platform_info)

    if session is None:
        return False, "No session to refresh"

    if session.is_expired:
        return False, "Session has expired and cannot be refreshed"

    # Update activity and extend expiration
    now = datetime.now(timezone.utc)
    session.last_activity = now
    session.expires_at = now + timedelta(hours=SESSION_TIMEOUT_HOURS)

    return save_session(session, platform_info)


def invalidate_session(
    platform_info: PlatformInfo | None = None,
) -> tuple[bool, str]:
    """
    Invalidate (logout) the current session.

    Args:
        platform_info: Optional platform info.

    Returns:
        Tuple of (success, message).
    """
    try:
        session_path = get_session_file_path(platform_info)

        if session_path.exists():
            session_path.unlink()
            return True, "Successfully logged out"

        return True, "No active session"

    except PermissionError:
        return False, "Permission denied when removing session"
    except Exception as e:
        return False, f"Failed to invalidate session: {e}"


def get_session_status_display(
    platform_info: PlatformInfo | None = None,
) -> str:
    """
    Get a formatted status display of the current session.

    Args:
        platform_info: Optional platform info.

    Returns:
        Formatted status string.
    """
    result = validate_session(platform_info)

    if result.status == SESSION_MISSING:
        return "Not logged in"

    if result.status == SESSION_EXPIRED:
        return f"Session expired (was: {result.session.username})"

    if result.status == SESSION_INVALID:
        return "Invalid session"

    # Valid session
    session = result.session
    hours_left = session.hours_remaining

    if hours_left < 1:
        minutes_left = int(session.time_remaining.total_seconds() / 60)
        time_str = f"{minutes_left} minutes"
    else:
        time_str = f"{hours_left:.1f} hours"

    return f"Logged in as {session.username} ({time_str} remaining)"


def needs_login(platform_info: PlatformInfo | None = None) -> bool:
    """
    Check if login is required.

    Args:
        platform_info: Optional platform info.

    Returns:
        True if login is needed.
    """
    result = validate_session(platform_info)
    return result.needs_login


def get_current_user(
    platform_info: PlatformInfo | None = None,
) -> tuple[str | None, str | None]:
    """
    Get current logged-in user info.

    Args:
        platform_info: Optional platform info.

    Returns:
        Tuple of (username, email) or (None, None) if not logged in.
    """
    result = validate_session(platform_info)

    if result.status == SESSION_VALID and result.session:
        return result.session.username, result.session.email

    return None, None


class SessionManager:
    """
    High-level session manager for the AiCippy CLI.

    Provides a convenient interface for session operations with
    automatic platform detection and caching.
    """

    def __init__(self, platform_info: PlatformInfo | None = None) -> None:
        """Initialize session manager."""
        self._platform_info = platform_info or detect_platform()
        self._cached_session: SessionInfo | None = None
        self._last_check: float = 0
        self._check_interval: float = 60.0  # Re-check every 60 seconds

    @property
    def platform_info(self) -> PlatformInfo:
        """Get platform information."""
        return self._platform_info

    def _should_recheck(self) -> bool:
        """Check if we should re-validate the session."""
        return time.monotonic() - self._last_check > self._check_interval

    def login(
        self,
        user_id: str,
        username: str,
        email: str,
        ip_address: str | None = None,
        metadata: dict[str, str] | None = None,
    ) -> tuple[bool, str]:
        """
        Create a new login session.

        Args:
            user_id: User's unique ID.
            username: User's display name.
            email: User's email.
            ip_address: Optional IP address.
            metadata: Optional metadata.

        Returns:
            Tuple of (success, message).
        """
        session = create_session(
            user_id=user_id,
            username=username,
            email=email,
            ip_address=ip_address,
            metadata=metadata,
        )

        success, message = save_session(session, self._platform_info)

        if success:
            self._cached_session = session
            self._last_check = time.monotonic()

        return success, message

    def logout(self) -> tuple[bool, str]:
        """
        Logout the current session.

        Returns:
            Tuple of (success, message).
        """
        success, message = invalidate_session(self._platform_info)

        if success:
            self._cached_session = None

        return success, message

    def validate(self) -> SessionValidationResult:
        """
        Validate the current session.

        Returns:
            SessionValidationResult with status.
        """
        # Use cached session if recent
        if self._cached_session and not self._should_recheck():
            if not self._cached_session.is_expired:
                return SessionValidationResult(
                    status=SESSION_VALID,
                    session=self._cached_session,
                    message=f"Welcome back, {self._cached_session.username}!",
                    needs_login=False,
                )

        # Full validation
        result = validate_session(self._platform_info)
        self._last_check = time.monotonic()

        if result.status == SESSION_VALID:
            self._cached_session = result.session
        else:
            self._cached_session = None

        return result

    def refresh(self) -> tuple[bool, str]:
        """
        Refresh the current session.

        Returns:
            Tuple of (success, message).
        """
        success, message = refresh_session(self._platform_info)

        if success:
            self._cached_session = load_session(self._platform_info)
            self._last_check = time.monotonic()

        return success, message

    def needs_login(self) -> bool:
        """
        Check if login is required.

        Returns:
            True if login needed.
        """
        return self.validate().needs_login

    def get_user(self) -> tuple[str | None, str | None]:
        """
        Get current user info.

        Returns:
            Tuple of (username, email).
        """
        result = self.validate()

        if result.status == SESSION_VALID and result.session:
            return result.session.username, result.session.email

        return None, None

    def get_status(self) -> str:
        """
        Get formatted session status.

        Returns:
            Status string.
        """
        return get_session_status_display(self._platform_info)

    def touch(self) -> None:
        """Update session activity without full refresh."""
        if self._cached_session and not self._cached_session.is_expired:
            self._cached_session.last_activity = datetime.now(timezone.utc)
            # Save periodically (every 5 minutes of activity)
            if self._should_recheck():
                self.refresh()



"""
Main Installer Orchestrator for AiCippy.

Coordinates the complete installation workflow including:
- Prerequisite checks and dependency installation
- Permission configuration
- Environment variable setup
- Version management and auto-updates
- Progress visualization
"""

from __future__ import annotations

import asyncio
import subprocess
import sys
from dataclasses import dataclass, field
from pathlib import Path
from typing import AsyncIterator, Callable, Final

from rich.console import Console

from aicippy.installer.dependency_manager import (
    check_dependencies,
    install_all_dependencies,
    verify_installation as verify_deps,
)
from aicippy.installer.env_manager import (
    get_env_export_commands,
    setup_all_env_variables,
    verify_env_variables,
)
from aicippy.installer.permission_manager import (
    setup_all_directories,
    verify_all_permissions,
)
from aicippy.installer.platform_detect import (
    MIN_PYTHON_VERSION,
    PlatformInfo,
    detect_platform,
)
from aicippy.installer.progress_ui import (
    INSTALLATION_PHASES,
    InstallationProgress,
)

# Version information
CURRENT_VERSION: Final[str] = "1.0.4"
VERSION_CHECK_URL: Final[str] = "https://api.aicippy.io/version"
PYPI_PACKAGE_NAME: Final[str] = "aicippy"


@dataclass
class InstallationResult:
    """Result of installation process."""

    success: bool
    version: str
    phases_completed: list[str]
    errors: list[str] = field(default_factory=list)
    warnings: list[str] = field(default_factory=list)
    requires_restart: bool = False
    env_commands: list[str] = field(default_factory=list)


@dataclass
class VersionInfo:
    """Version information."""

    current: str
    latest: str | None
    update_available: bool
    release_notes: str | None = None


async def check_latest_version() -> VersionInfo:
    """
    Check for the latest available version.

    Returns:
        VersionInfo with current and latest versions.
    """
    try:
        # Try PyPI first
        result = subprocess.run(
            [sys.executable, "-m", "pip", "index", "versions", PYPI_PACKAGE_NAME],
            capture_output=True,
            text=True,
            timeout=30,
        )

        if result.returncode == 0:
            # Parse output to find latest version
            # pip output format:
            #   aicippy (1.0.0)
            #   Available versions: 1.0.0, 0.9.0, ...
            #     INSTALLED: 1.0.0
            #     LATEST:    1.0.0
            output = result.stdout
            if "Available versions:" in output:
                # Get only the first line after "Available versions:"
                versions_section = output.split("Available versions:")[1]
                first_line = versions_section.split("\n")[0].strip()
                versions = [v.strip() for v in first_line.split(",") if v.strip()]
                if versions:
                    latest = versions[0]
                    # Only show update if versions actually differ
                    is_update = latest != CURRENT_VERSION
                    return VersionInfo(
                        current=CURRENT_VERSION,
                        latest=latest,
                        update_available=is_update,
                    )
    except Exception:
        pass

    # Fallback: assume current is latest
    return VersionInfo(
        current=CURRENT_VERSION,
        latest=CURRENT_VERSION,
        update_available=False,
    )


async def perform_upgrade() -> tuple[bool, str]:
    """
    Perform package upgrade.

    Returns:
        Tuple of (success, message).
    """
    try:
        result = subprocess.run(
            [sys.executable, "-m", "pip", "install", "--upgrade", PYPI_PACKAGE_NAME],
            capture_output=True,
            text=True,
            timeout=300,
        )

        if result.returncode == 0:
            return True, "Successfully upgraded AiCippy"
        else:
            return False, f"Upgrade failed: {result.stderr}"

    except subprocess.TimeoutExpired:
        return False, "Upgrade timed out"
    except Exception as e:
        return False, f"Upgrade error: {e}"


def check_prerequisites(platform_info: PlatformInfo) -> tuple[bool, list[str]]:
    """
    Check all prerequisites for installation.

    Args:
        platform_info: Platform information.

    Returns:
        Tuple of (all_met, list of issues).
    """
    issues: list[str] = []

    # Check Python version
    if not platform_info.python_meets_minimum:
        py_ver = platform_info.python_version
        min_ver = MIN_PYTHON_VERSION
        issues.append(
            f"Python {min_ver[0]}.{min_ver[1]}+ required, found {py_ver[0]}.{py_ver[1]}.{py_ver[2]}"
        )

    # Check pip availability
    if platform_info.pip_path is None:
        issues.append("pip not found. Please install pip first.")

    # Add any platform-specific errors
    issues.extend(platform_info.errors)

    return len(issues) == 0, issues


async def installation_phase_generator(
    platform_info: PlatformInfo,
    include_deps: bool = True,
) -> AsyncIterator[tuple[str, int, str]]:
    """
    Generate installation phases with progress updates.

    Args:
        platform_info: Platform information.
        include_deps: Whether to install dependencies.

    Yields:
        Tuples of (phase_id, progress_percent, status_message).
    """
    errors: list[str] = []
    warnings: list[str] = []

    # Phase 1: Initialization
    yield "init", 0, "Starting installation..."
    await asyncio.sleep(0.3)
    yield "init", 50, "Detecting platform..."
    await asyncio.sleep(0.2)
    yield "init", 100, f"Platform: {platform_info.os_name} ({platform_info.architecture.name})"

    # Phase 2: Prerequisites
    yield "prereq", 0, "Checking prerequisites..."
    prereq_ok, prereq_issues = check_prerequisites(platform_info)
    await asyncio.sleep(0.2)

    if not prereq_ok:
        for issue in prereq_issues:
            yield "prereq", 50, f"Issue: {issue}"
            errors.append(issue)
            await asyncio.sleep(0.3)

        if errors:
            yield "prereq", 100, "Prerequisites check failed"
            return

    yield "prereq", 100, "All prerequisites met"

    # Phase 3: Dependencies
    if include_deps:
        yield "deps", 0, "Checking dependencies..."
        dep_check = check_dependencies()
        await asyncio.sleep(0.2)

        if not dep_check.all_satisfied:
            missing = dep_check.missing_count
            upgrade = dep_check.upgrade_count
            total = missing + upgrade

            yield "deps", 10, f"Installing {total} package(s)..."

            def deps_progress(msg: str, pct: int) -> None:
                # This will be called synchronously but we handle it async
                pass

            success, dep_errors = install_all_dependencies(
                platform_info,
                progress_callback=deps_progress,
            )

            if not success:
                errors.extend(dep_errors)
                for err in dep_errors[:3]:
                    yield "deps", 50, f"Error: {err}"
                    await asyncio.sleep(0.2)

            # Simulate progress
            for pct in range(20, 101, 20):
                yield "deps", pct, f"Installing dependencies... {pct}%"
                await asyncio.sleep(0.3)
        else:
            yield "deps", 100, "All dependencies satisfied"
    else:
        yield "deps", 100, "Skipping dependency check"

    # Phase 4: Permissions
    yield "perms", 0, "Configuring permissions..."
    await asyncio.sleep(0.2)

    success, perm_results = setup_all_directories(platform_info)

    for i, result in enumerate(perm_results):
        pct = int((i + 1) / len(perm_results) * 100)
        if result.success:
            yield "perms", pct, f"Created: {result.path.name}"
        else:
            yield "perms", pct, f"Failed: {result.message}"
            errors.append(result.message)
        await asyncio.sleep(0.1)

    yield "perms", 100, "Permissions configured"

    # Phase 5: Environment
    yield "env", 0, "Setting up environment..."
    await asyncio.sleep(0.2)

    success, env_results, requires_restart = setup_all_env_variables(platform_info)

    for i, result in enumerate(env_results):
        pct = int((i + 1) / len(env_results) * 100)
        if result.success:
            yield "env", pct, f"Set: {result.name}"
        else:
            yield "env", pct, f"Failed: {result.message}"
            if result.error:
                warnings.append(f"{result.name}: {result.error}")
        await asyncio.sleep(0.1)

    if requires_restart:
        warnings.append("Shell restart required for PATH changes")

    yield "env", 100, "Environment configured"

    # Phase 6: Installation finalization
    yield "install", 0, "Finalizing installation..."
    await asyncio.sleep(0.3)

    yield "install", 50, "Registering commands..."
    await asyncio.sleep(0.2)

    yield "install", 100, "Installation complete"

    # Phase 7: Verification
    yield "verify", 0, "Verifying installation..."
    await asyncio.sleep(0.2)

    # Verify dependencies
    deps_ok, deps_msg = verify_deps()
    if deps_ok:
        yield "verify", 33, "Dependencies verified"
    else:
        yield "verify", 33, f"Warning: {deps_msg}"
        warnings.append(deps_msg)

    await asyncio.sleep(0.1)

    # Verify permissions
    perms_ok, perm_issues = verify_all_permissions(platform_info)
    if perms_ok:
        yield "verify", 66, "Permissions verified"
    else:
        for path, issue in perm_issues[:2]:
            yield "verify", 66, f"Warning: {issue}"
            warnings.append(issue)

    await asyncio.sleep(0.1)

    # Verify environment
    env_ok, missing_vars = verify_env_variables(platform_info)
    if env_ok:
        yield "verify", 100, "Environment verified"
    else:
        for var in missing_vars[:2]:
            yield "verify", 100, f"Note: {var} not set (restart may be needed)"

    await asyncio.sleep(0.1)
    yield "verify", 100, "Verification complete"


async def install_aicippy(
    console: Console | None = None,
    show_progress: bool = True,
    include_deps: bool = True,
) -> InstallationResult:
    """
    Perform complete AiCippy installation.

    Args:
        console: Optional Rich console.
        show_progress: Whether to show progress UI.
        include_deps: Whether to install dependencies.

    Returns:
        InstallationResult with status.
    """
    console = console or Console()
    platform_info = detect_platform()

    errors: list[str] = []
    warnings: list[str] = []
    phases_completed: list[str] = []
    requires_restart = False

    if show_progress:
        progress = InstallationProgress(console)

        # Show welcome
        progress.show_welcome()
        await asyncio.sleep(1)

        # Run with animated progress
        try:
            async def tracked_generator():
                nonlocal errors, warnings, phases_completed, requires_restart

                async for phase, pct, status in installation_phase_generator(
                    platform_info, include_deps
                ):
                    if phase not in phases_completed and pct == 100:
                        phases_completed.append(phase)
                    yield phase, pct, status

                # Check for restart requirement
                _, _, restart_needed = setup_all_env_variables.__wrapped__ if hasattr(
                    setup_all_env_variables, '__wrapped__'
                ) else (True, [], True)

            await progress.run_with_progress(tracked_generator())

        except Exception as e:
            errors.append(f"Installation error: {e}")

        # Show completion
        success = len(errors) == 0
        progress.show_completion(success=success, errors=errors if not success else None)

    else:
        # Silent installation
        async for phase, pct, status in installation_phase_generator(
            platform_info, include_deps
        ):
            if phase not in phases_completed and pct == 100:
                phases_completed.append(phase)

    # Get environment export commands for user reference
    env_commands = get_env_export_commands(platform_info)

    return InstallationResult(
        success=len(errors) == 0,
        version=CURRENT_VERSION,
        phases_completed=phases_completed,
        errors=errors,
        warnings=warnings,
        requires_restart=requires_restart,
        env_commands=env_commands,
    )


async def uninstall_aicippy(
    console: Console | None = None,
    remove_config: bool = False,
) -> tuple[bool, str]:
    """
    Uninstall AiCippy.

    Args:
        console: Optional Rich console.
        remove_config: Whether to remove configuration files.

    Returns:
        Tuple of (success, message).
    """
    console = console or Console()
    platform_info = detect_platform()

    try:
        # Uninstall package
        result = subprocess.run(
            [sys.executable, "-m", "pip", "uninstall", "-y", PYPI_PACKAGE_NAME],
            capture_output=True,
            text=True,
            timeout=120,
        )

        if result.returncode != 0:
            return False, f"Uninstall failed: {result.stderr}"

        # Optionally remove config
        if remove_config:
            import shutil

            config_path = platform_info.config_path
            cache_path = platform_info.cache_path

            if config_path.exists():
                shutil.rmtree(config_path, ignore_errors=True)

            if cache_path.exists():
                shutil.rmtree(cache_path, ignore_errors=True)

        return True, "AiCippy uninstalled successfully"

    except subprocess.TimeoutExpired:
        return False, "Uninstall timed out"
    except Exception as e:
        return False, f"Uninstall error: {e}"


async def upgrade_aicippy(
    console: Console | None = None,
    check_only: bool = False,
) -> tuple[bool, str, VersionInfo]:
    """
    Check for and optionally perform upgrade.

    Args:
        console: Optional Rich console.
        check_only: If True, only check for updates without installing.

    Returns:
        Tuple of (success, message, version_info).
    """
    console = console or Console()

    # Check version
    version_info = await check_latest_version()

    if not version_info.update_available:
        return True, f"AiCippy {version_info.current} is up to date", version_info

    if check_only:
        return True, f"Update available: {version_info.current} -> {version_info.latest}", version_info

    # Perform upgrade
    success, message = await perform_upgrade()

    if success:
        # Re-check version
        new_version = await check_latest_version()
        return True, f"Upgraded to {new_version.current}", new_version

    return False, message, version_info


def verify_installation() -> tuple[bool, list[str]]:
    """
    Verify that AiCippy is properly installed.

    Returns:
        Tuple of (success, list of issues).
    """
    issues: list[str] = []
    platform_info = detect_platform()

    # Check dependencies
    deps_ok, deps_msg = verify_deps()
    if not deps_ok:
        issues.append(f"Dependencies: {deps_msg}")

    # Check permissions
    perms_ok, perm_issues = verify_all_permissions(platform_info)
    if not perms_ok:
        for path, issue in perm_issues:
            issues.append(f"Permission: {issue}")

    # Check environment
    env_ok, missing_vars = verify_env_variables(platform_info)
    if not env_ok:
        for var in missing_vars:
            issues.append(f"Environment: {var}")

    # Check if aicippy command is accessible
    import shutil
    if shutil.which("aicippy") is None:
        issues.append("Command 'aicippy' not found in PATH (may need shell restart)")

    return len(issues) == 0, issues


def get_installation_info() -> dict:
    """
    Get information about the current installation.

    Returns:
        Dictionary with installation details.
    """
    platform_info = detect_platform()

    return {
        "version": CURRENT_VERSION,
        "python_version": ".".join(str(v) for v in platform_info.python_version),
        "platform": platform_info.os_name,
        "architecture": platform_info.architecture.name,
        "shell": platform_info.shell_type.name,
        "config_path": str(platform_info.config_path),
        "cache_path": str(platform_info.cache_path),
        "install_path": str(platform_info.user_install_path),
        "is_admin": platform_info.is_admin,
    }


# Entry point for direct execution
if __name__ == "__main__":
    async def main():
        result = await install_aicippy(show_progress=True)
        if result.success:
            print("\nInstallation completed successfully!")
        else:
            print("\nInstallation failed:")
            for error in result.errors:
                print(f"  - {error}")

    asyncio.run(main())



"""
AI Command Executor for AiCippy CLI.

Provides automatic shell command detection and execution with user input
passthrough when AI agents generate commands that need to be run.

Features:
- Parses AI responses for shell command blocks
- Detects suggested commands in various formats
- Executes commands with user confirmation
- Passes user input through for interactive commands
- Returns output to AI for context
"""

from __future__ import annotations

import asyncio
import re
from dataclasses import dataclass, field
from pathlib import Path
from typing import Callable, Final

from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm
from rich.syntax import Syntax
from rich.text import Text

from aicippy.cli.shell_executor import (
    CommandResult,
    ShellExecutor,
    is_interactive_command,
)
from aicippy.cli.upload_manager import get_working_directory

# Brand colors
BRAND_PRIMARY: Final[str] = "#667eea"
BRAND_SUCCESS: Final[str] = "#10b981"
BRAND_WARNING: Final[str] = "#f59e0b"
BRAND_ERROR: Final[str] = "#ef4444"
BRAND_INFO: Final[str] = "#3b82f6"
BRAND_ACCENT: Final[str] = "#a78bfa"

# Regex patterns for detecting command blocks
CODE_BLOCK_PATTERNS: Final[list[tuple[str, str]]] = [
    # Standard markdown code blocks with language
    (r"```(?:bash|sh|shell|zsh|fish)\n(.*?)```", "bash"),
    (r"```(?:cmd|powershell|ps1)\n(.*?)```", "shell"),
    # Generic code blocks that look like shell commands
    (r"```\n(\$.*?)```", "bash"),
    (r"```\n(>.*?)```", "shell"),
    # Inline code that looks like commands
    (r"`(\$ [^`]+)`", "bash"),
]

# Patterns that indicate a command suggestion
COMMAND_SUGGESTION_PATTERNS: Final[list[str]] = [
    r"(?:run|execute|try|use)(?: the)?(?: command)?:?\s*`([^`]+)`",
    r"you (?:can|should) run:?\s*`([^`]+)`",
    r"(?:run|execute) this:?\s*\n*```(?:bash|sh|shell)?\n(.*?)```",
    r"to (?:install|run|execute|start|deploy|build|test):?\s*\n*```(?:bash|sh|shell)?\n(.*?)```",
]


@dataclass
class DetectedCommand:
    """A command detected from AI response."""

    command: str
    language: str
    context: str  # Surrounding text that suggested this command
    is_suggestion: bool  # True if AI explicitly suggested running this
    line_number: int = 0


@dataclass
class CommandExecutionResult:
    """Result of AI command execution."""

    command: str
    result: CommandResult
    was_executed: bool
    user_declined: bool = False
    error: str = ""


@dataclass
class AICommandExecutionContext:
    """Context for AI command execution session."""

    commands_executed: list[CommandExecutionResult] = field(default_factory=list)
    total_commands_detected: int = 0
    total_commands_run: int = 0
    auto_execute: bool = False  # If True, execute without confirmation


class AICommandExecutor:
    """
    Detects and executes shell commands from AI responses.

    Parses AI output for command blocks and suggestions, prompts user
    for confirmation, and executes commands with input passthrough.
    """

    def __init__(
        self,
        console: Console | None = None,
        working_dir: Path | None = None,
        auto_confirm: bool = False,
    ) -> None:
        """
        Initialize AI command executor.

        Args:
            console: Rich console for output.
            working_dir: Working directory for commands.
            auto_confirm: Auto-confirm safe commands (read-only).
        """
        self.console = console or Console()
        self.working_dir = working_dir or get_working_directory()
        self.auto_confirm = auto_confirm
        self._shell_executor = ShellExecutor(
            console=self.console,
            working_dir=self.working_dir,
        )
        self._context = AICommandExecutionContext()

    def detect_commands(self, ai_response: str) -> list[DetectedCommand]:
        """
        Detect shell commands in AI response.

        Args:
            ai_response: The AI's response text.

        Returns:
            List of detected commands with context.
        """
        commands: list[DetectedCommand] = []
        seen_commands: set[str] = set()

        # First, find explicit code blocks
        for pattern, lang in CODE_BLOCK_PATTERNS:
            for match in re.finditer(pattern, ai_response, re.DOTALL | re.IGNORECASE):
                cmd_text = match.group(1).strip()
                # Clean up the command
                for line in cmd_text.split("\n"):
                    line = line.strip()
                    # Remove common prefixes
                    if line.startswith("$ "):
                        line = line[2:]
                    elif line.startswith("> "):
                        line = line[2:]

                    if line and line not in seen_commands and self._is_valid_command(line):
                        seen_commands.add(line)
                        # Get surrounding context
                        start = max(0, match.start() - 100)
                        end = min(len(ai_response), match.end() + 50)
                        context = ai_response[start:end]

                        commands.append(DetectedCommand(
                            command=line,
                            language=lang,
                            context=context,
                            is_suggestion=self._is_suggestion_context(context),
                            line_number=ai_response[:match.start()].count("\n") + 1,
                        ))

        # Then, find suggested commands
        for pattern in COMMAND_SUGGESTION_PATTERNS:
            for match in re.finditer(pattern, ai_response, re.DOTALL | re.IGNORECASE):
                cmd_text = match.group(1).strip()
                for line in cmd_text.split("\n"):
                    line = line.strip()
                    if line.startswith("$ "):
                        line = line[2:]
                    elif line.startswith("> "):
                        line = line[2:]

                    if line and line not in seen_commands and self._is_valid_command(line):
                        seen_commands.add(line)
                        commands.append(DetectedCommand(
                            command=line,
                            language="bash",
                            context=match.group(0),
                            is_suggestion=True,
                            line_number=ai_response[:match.start()].count("\n") + 1,
                        ))

        self._context.total_commands_detected += len(commands)
        return commands

    def _is_valid_command(self, cmd: str) -> bool:
        """Check if text looks like a valid shell command."""
        if not cmd or len(cmd) < 2:
            return False

        # Skip if it's just a comment
        if cmd.startswith("#"):
            return False

        # Skip if it's a variable assignment without a command
        if "=" in cmd and not any(c in cmd for c in [" ", "|", ";"]):
            return False

        # Must start with a word character (command name)
        if not re.match(r"^[a-zA-Z_/.]", cmd):
            return False

        return True

    def _is_suggestion_context(self, context: str) -> bool:
        """Check if context suggests the AI wants user to run this command."""
        suggestion_keywords = [
            "run", "execute", "try", "use", "install", "should",
            "can run", "following command", "this command",
            "to do this", "you can", "you should",
        ]
        context_lower = context.lower()
        return any(keyword in context_lower for keyword in suggestion_keywords)

    def _is_safe_command(self, cmd: str) -> bool:
        """Check if command is read-only/safe to auto-execute."""
        safe_prefixes = [
            "ls", "pwd", "echo", "cat", "head", "tail",
            "which", "where", "type", "file", "stat",
            "git status", "git log", "git diff", "git branch",
            "npm list", "pip list", "pip show",
            "node --version", "npm --version", "python --version",
            "aws sts get-caller-identity",
        ]
        cmd_lower = cmd.lower().strip()
        return any(cmd_lower.startswith(prefix) for prefix in safe_prefixes)

    def _is_dangerous_command(self, cmd: str) -> bool:
        """Check if command could be dangerous."""
        dangerous_patterns = [
            r"\brm\s+-rf",
            r"\brmdir\s+/",
            r"\bdel\s+/",
            r"\bformat\s+",
            r"\bmkfs\b",
            r"\bdd\s+",
            r">\s*/dev/",
            r"\bsudo\s+rm",
            r"\bchmod\s+777",
            r"\bcurl.*\|\s*(?:bash|sh)",
            r"\bwget.*\|\s*(?:bash|sh)",
            r":\(\)\{.*:\|:.*\}",  # Fork bomb
        ]
        for pattern in dangerous_patterns:
            if re.search(pattern, cmd, re.IGNORECASE):
                return True
        return False

    async def execute_detected_commands(
        self,
        ai_response: str,
        execute_all: bool = False,
        input_callback: Callable[[str], str] | None = None,
    ) -> list[CommandExecutionResult]:
        """
        Detect and execute commands from AI response.

        Args:
            ai_response: The AI's response text.
            execute_all: Execute all commands without individual prompts.
            input_callback: Callback for getting user input during execution.

        Returns:
            List of execution results.
        """
        commands = self.detect_commands(ai_response)

        if not commands:
            return []

        results: list[CommandExecutionResult] = []

        # Show summary of detected commands
        self.console.print(Panel(
            Text(
                f"Detected {len(commands)} command(s) in AI response.\n"
                f"Commands marked as suggestions will be highlighted.",
                style=BRAND_INFO,
            ),
            title=f"[bold {BRAND_ACCENT}]Commands Detected[/bold {BRAND_ACCENT}]",
            border_style=BRAND_ACCENT,
        ))

        for i, detected in enumerate(commands, 1):
            result = await self._execute_single_command(
                detected,
                index=i,
                total=len(commands),
                execute_all=execute_all,
                input_callback=input_callback,
            )
            results.append(result)
            self._context.commands_executed.append(result)

            if result.was_executed:
                self._context.total_commands_run += 1

        return results

    async def _execute_single_command(
        self,
        detected: DetectedCommand,
        index: int,
        total: int,
        execute_all: bool = False,
        input_callback: Callable[[str], str] | None = None,
    ) -> CommandExecutionResult:
        """Execute a single detected command with confirmation."""
        # Display command
        cmd_display = Syntax(
            detected.command,
            detected.language,
            theme="monokai",
            line_numbers=False,
        )

        # Build info text
        info_parts = []
        if detected.is_suggestion:
            info_parts.append("[bold green]AI suggested running this[/bold green]")
        if self._is_safe_command(detected.command):
            info_parts.append("[dim]Read-only/safe command[/dim]")
        if self._is_dangerous_command(detected.command):
            info_parts.append(f"[bold {BRAND_ERROR}]Potentially dangerous[/bold {BRAND_ERROR}]")

        info_text = " | ".join(info_parts) if info_parts else ""

        self.console.print(Panel(
            cmd_display,
            title=f"[bold]Command {index}/{total}[/bold] {info_text}",
            border_style=BRAND_WARNING if self._is_dangerous_command(detected.command) else BRAND_ACCENT,
        ))

        # Determine if we should auto-execute
        should_auto = (
            execute_all
            or self._context.auto_execute
            or (self.auto_confirm and self._is_safe_command(detected.command))
        )

        # Block dangerous commands from auto-execution
        if self._is_dangerous_command(detected.command):
            should_auto = False
            self.console.print(Text(
                "This command requires explicit confirmation due to potential risk.",
                style=BRAND_WARNING,
            ))

        # Ask for confirmation
        if not should_auto:
            try:
                should_execute = Confirm.ask(
                    "Execute this command?",
                    default=detected.is_suggestion,
                )
            except (EOFError, KeyboardInterrupt):
                return CommandExecutionResult(
                    command=detected.command,
                    result=CommandResult(
                        success=False,
                        exit_code=-1,
                        stdout="",
                        stderr="Cancelled by user",
                        command=detected.command,
                    ),
                    was_executed=False,
                    user_declined=True,
                )
        else:
            should_execute = True
            self.console.print(Text(
                "Auto-executing...",
                style="dim italic",
            ))

        if not should_execute:
            return CommandExecutionResult(
                command=detected.command,
                result=CommandResult(
                    success=False,
                    exit_code=-1,
                    stdout="",
                    stderr="User declined",
                    command=detected.command,
                ),
                was_executed=False,
                user_declined=True,
            )

        # Execute the command
        try:
            if is_interactive_command(detected.command):
                # Use interactive execution with passthrough
                def get_input(prompt: str) -> str:
                    if input_callback:
                        return input_callback(prompt)
                    return self._shell_executor.prompt_user_input(prompt)

                result = await self._shell_executor.execute_interactive(
                    detected.command,
                    input_callback=get_input,
                )
            else:
                # Use simple execution
                result = self._shell_executor.execute_simple(detected.command)

            # Display result
            self._shell_executor.display_result(result)

            return CommandExecutionResult(
                command=detected.command,
                result=result,
                was_executed=True,
            )

        except Exception as e:
            error_result = CommandResult(
                success=False,
                exit_code=-1,
                stdout="",
                stderr=str(e),
                command=detected.command,
            )
            return CommandExecutionResult(
                command=detected.command,
                result=error_result,
                was_executed=True,
                error=str(e),
            )

    def get_execution_summary(self) -> str:
        """Get summary of all command executions for AI context."""
        if not self._context.commands_executed:
            return ""

        parts = ["## Command Execution Summary\n"]
        for result in self._context.commands_executed:
            status = "executed" if result.was_executed else "skipped"
            if result.user_declined:
                status = "declined by user"

            parts.append(f"\n### `{result.command}`")
            parts.append(f"Status: {status}")

            if result.was_executed:
                parts.append(f"Exit code: {result.result.exit_code}")
                if result.result.stdout:
                    # Truncate long output
                    output = result.result.stdout[:1000]
                    if len(result.result.stdout) > 1000:
                        output += "\n... [truncated]"
                    parts.append(f"Output:\n```\n{output}\n```")
                if result.result.stderr:
                    parts.append(f"Errors:\n```\n{result.result.stderr[:500]}\n```")

        return "\n".join(parts)

    def clear_context(self) -> None:
        """Clear execution context for new session."""
        self._context = AICommandExecutionContext()


async def process_ai_response_with_commands(
    ai_response: str,
    console: Console | None = None,
    working_dir: Path | None = None,
    auto_execute_safe: bool = True,
) -> tuple[str, list[CommandExecutionResult]]:
    """
    Process an AI response, detect and execute any commands.

    Args:
        ai_response: The AI's response text.
        console: Rich console for output.
        working_dir: Working directory for commands.
        auto_execute_safe: Auto-execute safe/read-only commands.

    Returns:
        Tuple of (execution_summary, list of results).
    """
    executor = AICommandExecutor(
        console=console,
        working_dir=working_dir,
        auto_confirm=auto_execute_safe,
    )

    results = await executor.execute_detected_commands(ai_response)
    summary = executor.get_execution_summary()

    return summary, results



"""
WebSocket message models for AiCippy.
"""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Any


class MessageType(str, Enum):
    """Types of WebSocket messages."""

    # Client -> Server
    AUTHENTICATE = "authenticate"
    CHAT = "chat"
    TASK = "task"
    COMMAND = "command"
    PING = "ping"

    # Server -> Client
    AUTHENTICATED = "authenticated"
    RESPONSE = "response"
    AGENT_UPDATE = "agent_update"
    TOOL_OUTPUT = "tool_output"
    DIFF = "diff"
    PROGRESS = "progress"
    ERROR = "error"
    PONG = "pong"


@dataclass
class WebSocketMessage:
    """Base WebSocket message structure."""

    type: MessageType
    payload: dict[str, Any] = field(default_factory=dict)
    correlation_id: str | None = None
    timestamp: datetime = field(default_factory=datetime.utcnow)

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary for JSON serialization."""
        return {
            "type": self.type.value,
            "payload": self.payload,
            "correlation_id": self.correlation_id,
            "timestamp": self.timestamp.isoformat(),
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "WebSocketMessage":
        """Create from dictionary."""
        return cls(
            type=MessageType(data["type"]),
            payload=data.get("payload", {}),
            correlation_id=data.get("correlation_id"),
            timestamp=datetime.fromisoformat(data["timestamp"])
            if "timestamp" in data
            else datetime.utcnow(),
        )


@dataclass
class AgentUpdate:
    """Agent status update message."""

    agent_id: str
    agent_type: str
    status: str
    progress: int
    message: str = ""
    tokens_used: int = 0

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        return {
            "agent_id": self.agent_id,
            "agent_type": self.agent_type,
            "status": self.status,
            "progress": self.progress,
            "message": self.message,
            "tokens_used": self.tokens_used,
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "AgentUpdate":
        """Create from dictionary."""
        return cls(
            agent_id=data["agent_id"],
            agent_type=data["agent_type"],
            status=data["status"],
            progress=data["progress"],
            message=data.get("message", ""),
            tokens_used=data.get("tokens_used", 0),
        )


@dataclass
class ToolOutput:
    """Tool execution output message."""

    tool_name: str
    output: str
    success: bool
    execution_time_ms: int = 0
    metadata: dict[str, Any] = field(default_factory=dict)

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        return {
            "tool_name": self.tool_name,
            "output": self.output,
            "success": self.success,
            "execution_time_ms": self.execution_time_ms,
            "metadata": self.metadata,
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "ToolOutput":
        """Create from dictionary."""
        return cls(
            tool_name=data["tool_name"],
            output=data["output"],
            success=data["success"],
            execution_time_ms=data.get("execution_time_ms", 0),
            metadata=data.get("metadata", {}),
        )


@dataclass
class DiffMessage:
    """File diff message for code changes."""

    file_path: str
    old_content: str
    new_content: str
    diff_type: str = "unified"  # unified, side_by_side
    language: str | None = None

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        return {
            "file_path": self.file_path,
            "old_content": self.old_content,
            "new_content": self.new_content,
            "diff_type": self.diff_type,
            "language": self.language,
        }


@dataclass
class ProgressMessage:
    """Progress update message."""

    task_id: str
    current: int
    total: int
    message: str = ""
    stage: str | None = None

    @property
    def percentage(self) -> float:
        """Calculate percentage complete."""
        if self.total == 0:
            return 0.0
        return (self.current / self.total) * 100

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary."""
        return {
            "task_id": self.task_id,
            "current": self.current,
            "total": self.total,
            "message": self.message,
            "stage": self.stage,
            "percentage": self.percentage,
        }
